from django import forms
from django.forms.widgets import CheckboxSelectMultiple
from oscar.core.loading import import_module


class SimpleSearch(forms.Form):
    
    CHOICES=(('shipping_address', 'Shipping address'),
             ('billing_address', 'Billing address'), ('customer', 'Customer'))
    
    search_query = forms.CharField(max_length=64, label="Search for", required=True)
    search_by = forms.MultipleChoiceField(label="Using", choices=CHOICES, widget=CheckboxSelectMultiple(), required=False)



import mock
from decimal import Decimal as D
from xml.dom.minidom import parseString

from django.test import TestCase

from oscar.apps.payment.datacash.utils import Gateway


class TransactionMixin(object):
    
    def init_gateway(self, **kwargs):
        self.gateway = Gateway(client="DUMMY", password="123456", host="dummyhost.com", **kwargs)
        self.transport = mock.Mock()
        self.gateway.do_request = self.transport
        
        # Set a default success response
        response_xml = """<?xml version="1.0" encoding="UTF-8" ?>
<Response>
    <CardTxn>
        <authcode>060642</authcode>
        <card_scheme>Switch</card_scheme>
        <country>United Kingdom</country>
        <issuer>HSBC</issuer>
    </CardTxn>
    <datacash_reference>3000000088888888</datacash_reference>
    <merchantreference>1000001</merchantreference>
    <mode>LIVE</mode>
    <reason>ACCEPTED</reason>
    <status>1</status>
    <time>1071567305</time>
</Response>"""
        self.transport.return_value = response_xml
    
    def assertXmlTagValue(self, tag_name, value):
        request_xml = self.transport.call_args[0][0]
        doc = parseString(request_xml)
        try:
            tag = doc.getElementsByTagName(tag_name)[0]
            self.assertEquals(value, tag.firstChild.data)
        except IndexError:
            self.fail("Tag '%s' not found\n%s" % (tag_name, request_xml))
    
    def make_request(self, **kwargs):
        """
        Needs to be implemented by subclass.
        """
        pass
            
    def assertXmlTagAttributeValue(self, tag_name, attribute, value):
        request_xml = self.transport.call_args[0][0]
        doc = parseString(request_xml)
        try:
            tag = doc.getElementsByTagName(tag_name)[0]
            self.assertEquals(value, tag.attributes[attribute].value)
        except IndexError:
            self.fail("Tag '%s' not found\n%s" % (tag_name, request_xml))
    
    def test_request_includes_credentials(self):
        self.make_request()
        self.assertTrue(self.transport.called)
        self.assertXmlTagValue('client', 'DUMMY')
        self.assertXmlTagValue('password', '123456')


class InitialTransactionMixin(TransactionMixin):
    
    def test_request_includes_merchant_reference(self):   
        self.make_request()
        self.assertXmlTagValue('merchantreference', '123123')
        
    def test_request_includes_amount_and_currency(self):   
        self.make_request()
        self.assertXmlTagValue('amount', '12.99')
        self.assertXmlTagAttributeValue('amount', 'currency', 'GBP')
        
    def test_request_can_include_authcode(self):
        self.make_request(auth_code='334455')    
        self.assertXmlTagValue('authcode', '334455')
    
    def test_request_includes_card_details(self):   
        self.make_request(start_date='01/10')
        self.assertXmlTagValue('pan', '1000010000000007')
        self.assertXmlTagValue('startdate', '01/10')
        self.assertXmlTagValue('expirydate', '01/12')
        
    def test_request_can_include_issue_number(self):
        self.make_request(issue_number='03')    
        self.assertXmlTagValue('issuenumber', '03')    


class AuthTransactionTests(TestCase, InitialTransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.auth(card_number='1000010000000007', 
                          expiry_date='01/12',
                          merchant_reference='123123',
                          currency='GBP',
                          amount=D('12.99'),
                          **kwargs)
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'auth')
    
    
class PreTransactionTests(TestCase, InitialTransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.pre(card_number='1000010000000007', 
                         expiry_date='01/12',
                         merchant_reference='123123',
                         currency='GBP',
                         amount=D('12.99'),
                         **kwargs)    
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'pre')
        
        
class RefundTransactionTests(TestCase, InitialTransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.refund(card_number='1000010000000007', 
                            expiry_date='01/12',
                            merchant_reference='123123',
                            currency='GBP',
                            amount=D('12.99'),
                            **kwargs)    
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'refund')
    
    
class ErpTransactionTests(TestCase, InitialTransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.erp(card_number='1000010000000007', 
                         expiry_date='01/12',
                         merchant_reference='123123',
                         currency='GBP',
                         amount=D('12.99'),
                         **kwargs)    
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'erp')    
    
    
class CancelTransactionTests(TestCase, TransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.cancel(txn_reference='12312333444')
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'cancel') 
        
    def test_request_includes_reference(self):
        self.make_request()    
        self.assertXmlTagValue('reference', '12312333444') 
        
        
class FulfilTransactionTests(TestCase, TransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.fulfil(currency='GBP',
                            amount=D('12.99'),
                            txn_reference='12312333444',
                            auth_code='123444')
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'fulfil')   
        
    def test_request_includes_reference(self):
        self.make_request()    
        self.assertXmlTagValue('reference', '12312333444') 
        
    def test_request_includes_authcode(self):
        self.make_request()    
        self.assertXmlTagValue('authcode', '123444') 
        
        
class TxnRefundTransactionTests(TestCase, TransactionMixin):
    
    def setUp(self):
        self.init_gateway()
    
    def make_request(self, **kwargs):
        self.gateway.txn_refund(currency='GBP',
                                amount=D('12.99'),
                                txn_reference='12312333444')
        
    def test_request_includes_method(self):
        self.make_request()    
        self.assertXmlTagValue('method', 'txn_refund')   
        
    def test_request_includes_reference(self):
        self.make_request()    
        self.assertXmlTagValue('reference', '12312333444') 
        
        
class Av2CVsTests(TestCase, TransactionMixin):
    
    def setUp(self):
        self.init_gateway(cv2avs=True)
    
    def make_request(self, **kwargs):
        self.gateway.auth(card_number='1000010000000007', 
                          expiry_date='01/12',
                          merchant_reference='123123',
                          currency='GBP',
                          amount=D('12.99'),
                          ccv='234')
        
    def test_ccv_element_in_request(self):
        self.make_request()
        self.assertXmlTagValue('cv2', '234') 
    


import os

# Django settings for oscar project.
PROJECT_DIR = os.path.dirname(__file__)
location = lambda x: os.path.join(os.path.dirname(os.path.realpath(__file__)), x)

DEBUG = True
TEMPLATE_DEBUG = True
SQL_DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.', # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': '',                      # Or path to database file if using sqlite3.
        'USER': '',                      # Not used with sqlite3.
        'PASSWORD': '',                  # Not used with sqlite3.
        'HOST': '',                      # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                      # Set to empty string for default. Not used with sqlite3.
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/London'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = location("assets")

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/admin/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '$)a7n&o80u!6y5t-+jrd3)3!%vh&shg$wqpjpxc!ar&p#!)n1a'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.request",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.contrib.messages.context_processors.messages",
    # Oscar specific
    'oscar.apps.search.context_processors.search_form',
    'oscar.apps.promotions.context_processors.promotions',
    'oscar.apps.promotions.context_processors.merchandising_blocks',
) 

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    #'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
    'django.contrib.flatpages.middleware.FlatpageFallbackMiddleware'
)

INTERNAL_IPS = ('127.0.0.1',)

ROOT_URLCONF = 'urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    location("templates")
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'null': {
            'level':'DEBUG',
            'class':'django.utils.log.NullHandler',
        },
        'console':{
            'level':'DEBUG',
            'class':'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'file': {
             'level': 'INFO',
             'class': 'logging.FileHandler',
             'filename': '/tmp/oscar.log',
             'formatter': 'verbose'
        },
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler',
        },
    },
    'loggers': {
        'django': {
            'handlers':['null'],
            'propagate': True,
            'level':'INFO',
        },
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': False,
        },
        'oscar.checkout': {
            'handlers': ['console', 'file'],
            'propagate': True,
            'level':'INFO',
        },
        'django.db.backends': {
            'handlers':['null'],
            'propagate': False,
            'level':'DEBUG',
        },
    }
}


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.flatpages',
    # External apps
    'django_extensions',
    'haystack',
    #'debug_toolbar',
    # Apps from oscar
    'oscar',
    'oscar.apps.analytics',
    'oscar.apps.discount',
    'oscar.apps.order',
    'shop.checkout',
    'oscar.apps.shipping',
    'oscar.apps.order_management',
    'shop.product', # Override
    'oscar.apps.partner',
    'oscar.apps.basket',
    'oscar.apps.payment',
    'oscar.apps.offer',
    'oscar.apps.address',
    'oscar.apps.partner',
    'oscar.apps.image',
    'oscar.apps.customer',
    'oscar.apps.promotions',
    'oscar.apps.reports',
    'oscar.apps.search',
    'oscar.apps.catalogue_import',
    'oscar.apps.payment.datacash',
    'pyzen',
)

LOGIN_REDIRECT_URL = '/accounts/profile/'
APPEND_SLASH = True

# Oscar settings
from oscar.defaults import *

# Haystack settings
HAYSTACK_SITECONF = 'oscar.search_sites'
HAYSTACK_SEARCH_ENGINE = 'solr'
HAYSTACK_SOLR_URL = 'http://127.0.0.1:8080/solr'
HAYSTACK_INCLUDE_SPELLING = True

# Local overrides
try:
    from settings_local import *
    DATABASES['default']['NAME'] = "oscar_demo"
except ImportError:
    pass


from decimal import Decimal
import datetime

from django.conf import settings
from django.test import TestCase

from oscar.apps.offer.models import (Range, CountCondition, ValueCondition,
                                     CoverageCondition, ConditionalOffer,
                                     PercentageDiscountBenefit, FixedPriceBenefit,
                                     MultibuyDiscountBenefit, AbsoluteDiscountBenefit)
from oscar.apps.basket.models import Basket
from oscar.test.helpers import create_product


class RangeTest(TestCase):
    
    def setUp(self):
        self.prod = create_product()
    
    def test_all_products_range(self):
        range = Range.objects.create(name="All products", includes_all_products=True)
        self.assertTrue(range.contains_product(self.prod))
        
    def test_all_products_range_with_exception(self):
        range = Range.objects.create(name="All products", includes_all_products=True)
        range.excluded_products.add(self.prod)
        self.assertFalse(range.contains_product(self.prod))
        
    def test_empty_list(self):
        range = Range.objects.create(name="All products")
        self.assertFalse(range.contains_product(self.prod))
        
    def test_whitelisting(self):
        range = Range.objects.create(name="All products")
        range.included_products.add(self.prod)
        self.assertTrue(range.contains_product(self.prod))
        
    def test_blacklisting(self):
        range = Range.objects.create(name="All products", includes_all_products=True)
        range.excluded_products.add(self.prod)
        self.assertFalse(range.contains_product(self.prod))
        
    def test_included_classes(self):
        range = Range.objects.create(name="All products", includes_all_products=False)
        range.classes.add(self.prod.product_class)
        self.assertTrue(range.contains_product(self.prod))
        
    def test_included_class_with_exception(self):
        range = Range.objects.create(name="All products", includes_all_products=False)
        range.classes.add(self.prod.product_class)
        range.excluded_products.add(self.prod)
        self.assertFalse(range.contains_product(self.prod))


class OfferTest(TestCase):
    def setUp(self):
        self.range = Range.objects.create(name="All products range", includes_all_products=True)
        self.basket = Basket.objects.create()


class CountConditionTest(OfferTest):
    
    def setUp(self):
        super(CountConditionTest, self).setUp()
        self.cond = CountCondition(range=self.range, type="Count", value=2)
    
    def test_empty_basket_fails_condition(self):
        self.assertFalse(self.cond.is_satisfied(self.basket))
        
    def test_matching_quantity_basket_passes_condition(self):
        self.basket.add_product(create_product(), 2)
        self.assertTrue(self.cond.is_satisfied(self.basket))
        
    def test_greater_quantity_basket_passes_condition(self):
        self.basket.add_product(create_product(), 3)
        self.assertTrue(self.cond.is_satisfied(self.basket))

    def test_consumption(self):
        self.basket.add_product(create_product(), 3)
        self.cond.consume_items(self.basket)
        self.assertEquals(1, self.basket.all_lines()[0].quantity_without_discount)
        
    def test_is_satisfied_accounts_for_consumed_items(self):
        self.basket.add_product(create_product(), 3)
        self.cond.consume_items(self.basket)
        self.assertFalse(self.cond.is_satisfied(self.basket))
        
    def test_count_condition_is_applied_multpile_times(self):
        benefit = AbsoluteDiscountBenefit(range=self.range, type="Absolute", value=Decimal('10.00'))
        for i in range(10):
            self.basket.add_product(create_product(price=Decimal('5.00'), upc='upc_%i' % i), 1)
        product_range = Range.objects.create(name="All products", includes_all_products=True)
        condition = CountCondition(range=product_range, type="Count", value=2)

        first_discount = benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('10.00'), first_discount)
        
        second_discount = benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('10.00'), second_discount)

    
class ValueConditionTest(OfferTest):
    def setUp(self):
        super(ValueConditionTest, self).setUp()
        self.cond = ValueCondition(range=self.range, type="Value", value=Decimal('10.00'))
        self.item = create_product(price=Decimal('5.00'))
        self.expensive_item = create_product(price=Decimal('15.00'))
    
    def test_empty_basket_fails_condition(self):
        self.assertFalse(self.cond.is_satisfied(self.basket))
        
    def test_less_value_basket_fails_condition(self):
        self.basket.add_product(self.item, 1)
        self.assertFalse(self.cond.is_satisfied(self.basket))    
        
    def test_matching_basket_passes_condition(self):
        self.basket.add_product(self.item, 2)
        self.assertTrue(self.cond.is_satisfied(self.basket))   
        
    def test_greater_than_basket_passes_condition(self):
        self.basket.add_product(self.item, 3)
        self.assertTrue(self.cond.is_satisfied(self.basket)) 
        
    def test_consumption(self):
        self.basket.add_product(self.item, 3)
        self.cond.consume_items(self.basket)
        self.assertEquals(1, self.basket.all_lines()[0].quantity_without_discount)

    def test_consumption_with_high_value_product(self):
        self.basket.add_product(self.expensive_item, 1)
        self.cond.consume_items(self.basket)
        self.assertEquals(0, self.basket.all_lines()[0].quantity_without_discount)
        
    def test_is_consumed_respects_quantity_consumed(self):
        self.basket.add_product(self.expensive_item, 1)
        self.assertTrue(self.cond.is_satisfied(self.basket))
        self.cond.consume_items(self.basket)
        self.assertFalse(self.cond.is_satisfied(self.basket))

      
class CoverageConditionTest(TestCase):
    
    def setUp(self):
        self.products = [create_product(Decimal('5.00')), create_product(Decimal('10.00'))]
        self.range = Range.objects.create(name="Some products")
        for product in self.products:
            self.range.included_products.add(product)
            self.range.included_products.add(product)
            
        self.basket = Basket.objects.create()
        self.cond = CoverageCondition(range=self.range, type="Coverage", value=2)
    
    def test_empty_basket_fails(self):
        self.assertFalse(self.cond.is_satisfied(self.basket))
        
    def test_single_item_fails(self):
        self.basket.add_product(self.products[0])
        self.assertFalse(self.cond.is_satisfied(self.basket))
        
    def test_duplicate_item_fails(self):
        self.basket.add_product(self.products[0])
        self.basket.add_product(self.products[0])
        self.assertFalse(self.cond.is_satisfied(self.basket))  
        
    def test_covering_items_pass(self):
        self.basket.add_product(self.products[0])
        self.basket.add_product(self.products[1])
        self.assertTrue(self.cond.is_satisfied(self.basket))
        
    def test_covering_items_are_consumed(self):
        self.basket.add_product(self.products[0])
        self.basket.add_product(self.products[1])
        self.cond.consume_items(self.basket)
        self.assertEquals(0, self.basket.num_items_without_discount)
        
    def test_consumed_items_checks_affected_items(self):
        # Create new offer
        range = Range.objects.create(name="All products", includes_all_products=True)
        cond = CoverageCondition(range=range, type="Coverage", value=2)
        
        # Get 4 distinct products in the basket
        self.products.extend([create_product(Decimal('15.00')), create_product(Decimal('20.00'))])

        for product in self.products:        
            self.basket.add_product(product)
        
        self.assertTrue(cond.is_satisfied(self.basket))    
        cond.consume_items(self.basket)
        self.assertEquals(2, self.basket.num_items_without_discount) 
        
        self.assertTrue(cond.is_satisfied(self.basket))    
        cond.consume_items(self.basket)
        self.assertEquals(0, self.basket.num_items_without_discount) 
        
        
class PercentageDiscountBenefitTest(OfferTest):
    
    def setUp(self):
        super(PercentageDiscountBenefitTest, self).setUp()
        self.benefit = PercentageDiscountBenefit(range=self.range, type="Percentage", value=Decimal('15.00'))
        self.item = create_product(price=Decimal('5.00'))
        self.original_offer_rounding_function = getattr(settings, 'OSCAR_OFFER_ROUNDING_FUNCTION', None)
        if self.original_offer_rounding_function is not None:
            delattr(settings, 'OSCAR_OFFER_ROUNDING_FUNCTION')

    def tearDown(self):
        super(PercentageDiscountBenefitTest, self).tearDown()
        if self.original_offer_rounding_function is not None:
            settings.OSCAR_OFFER_ROUNDING_FUNCTION = self.original_offer_rounding_function
    
    def test_no_discount_for_empty_basket(self):
        self.assertEquals(Decimal('0.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_single_item_basket(self):
        self.basket.add_product(self.item, 1)
        self.assertEquals(Decimal('0.15') * Decimal('5.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_multi_item_basket(self):
        self.basket.add_product(self.item, 3)
        self.assertEquals(Decimal('3') * Decimal('0.15') * Decimal('5.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_multi_item_basket_with_max_affected_items_set(self):
        self.basket.add_product(self.item, 3)
        self.benefit.max_affected_items = 1
        self.assertEquals(Decimal('0.15') * Decimal('5.00'), self.benefit.apply(self.basket))
        
    def test_discount_can_only_be_applied_once(self):
        self.basket.add_product(self.item, 3)
        first_discount = self.benefit.apply(self.basket)
        second_discount = self.benefit.apply(self.basket)
        self.assertEquals(Decimal('0.00'), second_discount)
        
    def test_discount_can_be_applied_several_times_when_max_is_set(self):
        self.basket.add_product(self.item, 3)
        self.benefit.max_affected_items = 1
        for i in range(1, 4):
            self.assertTrue(self.benefit.apply(self.basket) > 0)
        
        
class AbsoluteDiscountBenefitTest(OfferTest):
    
    def setUp(self):
        super(AbsoluteDiscountBenefitTest, self).setUp()
        self.benefit = AbsoluteDiscountBenefit(range=self.range, type="Absolute", value=Decimal('10.00'))
        self.item = create_product(price=Decimal('5.00'))
        self.original_offer_rounding_function = getattr(settings, 'OSCAR_OFFER_ROUNDING_FUNCTION', None)
        if self.original_offer_rounding_function is not None:
            delattr(settings, 'OSCAR_OFFER_ROUNDING_FUNCTION')

    def tearDown(self):
        super(AbsoluteDiscountBenefitTest, self).tearDown()
        if self.original_offer_rounding_function is not None:
            settings.OSCAR_OFFER_ROUNDING_FUNCTION = self.original_offer_rounding_function
    
    def test_no_discount_for_empty_basket(self):
        self.assertEquals(Decimal('0.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_single_item_basket(self):
        self.basket.add_product(self.item, 1)
        self.assertEquals(Decimal('5.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_multi_item_basket(self):
        self.basket.add_product(self.item, 3)
        self.assertEquals(Decimal('10.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_multi_item_basket_with_max_affected_items_set(self):
        self.basket.add_product(self.item, 3)
        self.benefit.max_affected_items = 1
        self.assertEquals(Decimal('5.00'), self.benefit.apply(self.basket))

    # I'm not sure how to fix this
    #def test_discount_can_only_be_applied_once(self):
    #    # Add 3 items to make total 15.00
    #    self.basket.add_product(self.item, 3)
    #    first_discount = self.benefit.apply(self.basket)
    #    self.assertEquals(Decimal('10.00'), first_discount)
    #    
    #    second_discount = self.benefit.apply(self.basket)
    #    self.assertEquals(Decimal('5.00'), second_discount)
        
        
class MultibuyDiscountBenefitTest(OfferTest):
    
    def setUp(self):
        super(MultibuyDiscountBenefitTest, self).setUp()
        self.benefit = MultibuyDiscountBenefit(range=self.range, type="Multibuy", value=1)
        self.item = create_product(price=Decimal('5.00'))
    
    def test_no_discount_for_empty_basket(self):
        self.assertEquals(Decimal('0.00'), self.benefit.apply(self.basket))
        
    def test_discount_for_single_item_basket(self):
        self.basket.add_product(self.item, 1)
        self.assertEquals(Decimal('5.00'), self.benefit.apply(self.basket)) 
        
    def test_discount_for_multi_item_basket(self):
        self.basket.add_product(self.item, 3)
        self.assertEquals(Decimal('5.00'), self.benefit.apply(self.basket))   
        
    def test_discount_does_not_consume_item_if_in_condition_range(self):
        self.basket.add_product(self.item, 1)
        first_discount = self.benefit.apply(self.basket)
        self.assertEquals(Decimal('5.00'), first_discount)
        second_discount = self.benefit.apply(self.basket)
        self.assertEquals(Decimal('5.00'), second_discount)
        
    def test_product_does_consume_item_if_not_in_condition_range(self):
        # Set up condition using a different range from benefit
        range = Range.objects.create(name="Small range")
        other_product = create_product(price=Decimal('15.00'))
        range.included_products.add(other_product)
        cond = ValueCondition(range=range, type="Value", value=Decimal('10.00'))
        
        self.basket.add_product(self.item, 1)
        self.benefit.apply(self.basket, cond)
        line = self.basket.all_lines()[0]
        self.assertEqual(line.quantity_without_discount, 0)
        
    def test_condition_consumes_most_expensive_lines_first(self):
        for i in range(10, 0, -1):
            product = create_product(price=Decimal(i), title='%i'%i, upc='upc_%i' % i)
            self.basket.add_product(product, 1)

        condition = CountCondition(range=self.range, type="Count", value=2)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 1 and 10
        first_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('1.00'), first_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 2 and 9
        second_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('2.00'), second_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 3 and 8
        third_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('3.00'), third_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 4 and 7
        fourth_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('4.00'), fourth_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 5 and 6
        fifth_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('5.00'), fifth_discount)

        # end of items (one not discounted item in basket)
        self.assertFalse(condition.is_satisfied(self.basket))

    def test_condition_consumes_most_expensive_lines_first_when_products_are_repeated(self):
        for i in range(5, 0, -1):
            product = create_product(price=Decimal(i), title='%i'%i, upc='upc_%i' % i)
            self.basket.add_product(product, 2)

        condition = CountCondition(range=self.range, type="Count", value=2)

        # initial basket: [(1, 1), (2, 2), (3, 3), (4, 4), (5, 5)]
        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 1 and 5
        first_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('1.00'), first_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 1 and 5
        second_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('1.00'), second_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 2 and 4
        third_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('2.00'), third_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 2 and 4
        third_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('2.00'), third_discount)

        self.assertTrue(condition.is_satisfied(self.basket))
        # consume 3 and 3
        third_discount = self.benefit.apply(self.basket, condition=condition)
        self.assertEquals(Decimal('3.00'), third_discount)

        # end of items (one not discounted item in basket)
        self.assertFalse(condition.is_satisfied(self.basket))

class FixedPriceBenefitTest(OfferTest):
    
    def setUp(self):
        super(FixedPriceBenefitTest, self).setUp()
        self.benefit = FixedPriceBenefit(range=self.range, type="FixedPrice", value=Decimal('10.00'))
        
    def test_correct_discount_is_returned(self):
        products = [create_product(Decimal('8.00')), create_product(Decimal('4.00'))]
        range = Range.objects.create(name="Dummy range")
        for product in products:
            range.included_products.add(product)
            range.included_products.add(product)
            
        basket = Basket.objects.create()
        [basket.add_product(p) for p in products]
        
        condition = CoverageCondition(range=range, type="Coverage", value=2)
        discount = self.benefit.apply(basket, condition)
        self.assertEquals(Decimal('2.00'), discount)   
        
    def test_no_discount_is_returned_when_value_is_greater_than_product_total(self):
        products = [create_product(Decimal('4.00')), create_product(Decimal('4.00'))]
        range = Range.objects.create(name="Dummy range")
        for product in products:
            range.included_products.add(product)
            range.included_products.add(product)
            
        basket = Basket.objects.create()
        [basket.add_product(p) for p in products]
        
        condition = CoverageCondition(range=range, type="Coverage", value=2)
        discount = self.benefit.apply(basket, condition)
        self.assertEquals(Decimal('0.00'), discount) 
         
        
    
class ConditionalOfferTest(TestCase):
   
    def test_is_active(self):
        start = datetime.date(2011, 01, 01)
        test = datetime.date(2011, 01, 10)
        end = datetime.date(2011, 02, 01)
        offer = ConditionalOffer(start_date=start, end_date=end)
        self.assertTrue(offer.is_active(test))
       
    def test_is_inactive(self):
        start = datetime.date(2011, 01, 01)
        test = datetime.date(2011, 03, 10)
        end = datetime.date(2011, 02, 01)
        offer = ConditionalOffer(start_date=start, end_date=end)
        self.assertFalse(offer.is_active(test))
        

    
   



from django.db import models

from oscar.apps.shipping.abstract_models import AbstractOrderAndItemLevelChargeMethod


class OrderAndItemLevelChargeMethod(AbstractOrderAndItemLevelChargeMethod):
    pass


from django.db import models
from django.utils.translation import gettext as _
from django.core.exceptions import ValidationError
from django.conf import settings

from django.db.models import Sum, Count

from oscar.apps.catalogue.reviews.managers import (ApprovedReviewsManager, RecentReviewsManager, 
                                                   TopScoredReviewsManager, TopVotedReviewsManager)


class AbstractProductReview(models.Model):
    """
    Superclass ProductReview. Some key aspects have been implemented from the original spec.
    * Each product can have reviews attached to it. Each review has a title, a body and a score from 1-5.
    * Signed in users can always submit reviews, anonymous users can only submit reviews if a setting
      OSCAR_ALLOW_ANON_REVIEWS is set to true - it should default to false.
    * If anon users can submit reviews, then we require their name, email address and an (optional) URL.
    * By default, reviews must be approved before they are live.
      However, if a setting OSCAR_MODERATE_REVIEWS is set to false, then they don't need moderation.
    * Each review should have a permalink, ie it has its own page.
    * Each reviews can be voted up or down by other users
    * Only signed in users can vote
    * A user can only vote once on each product once
    """
    
    # Note we keep the review even if the product is deleted
    product = models.ForeignKey('catalogue.Product', related_name='reviews', null=True, on_delete=models.SET_NULL)
    
    SCORE_CHOICES = tuple([(x, x) for x in range(0, 6)])
    score = models.SmallIntegerField(_("Score"), choices=SCORE_CHOICES)
    title = models.CharField(_("Title"), max_length=255)
    body = models.TextField(_("Body"))
    
    # User information.  We include fields to handle anonymous users
    user = models.ForeignKey('auth.User', related_name='reviews', null=True, blank=True)
    name = models.CharField(_("Name"), max_length=255, null=True, blank=True)
    email = models.EmailField(_("Email"), null=True, blank=True)
    homepage = models.URLField(_("URL"), null=True, blank=True)
    
    FOR_MODERATION, APPROVED, REJECTED = range(0, 3)
    STATUS_CHOICES = (
        (FOR_MODERATION, _("Requires moderation")),
        (APPROVED, _("Approved")),
        (REJECTED, _("Rejected")), 
    ) 
    default_status = FOR_MODERATION if settings.OSCAR_MODERATE_REVIEWS else APPROVED
    status = models.SmallIntegerField(_("Status"), choices=STATUS_CHOICES, default=default_status)
    
    # Denormalised vote totals
    total_votes = models.IntegerField(_("Total Votes"), default=0)  # upvotes + down votes
    delta_votes = models.IntegerField(_("Delta Votes"), default=0, db_index=True)  # upvotes - down votes  
    
    date_created = models.DateTimeField(auto_now_add=True)
    
    # Managers
    objects = models.Manager()
    approved = ApprovedReviewsManager()

    class Meta:
        abstract = True
        ordering = ['-delta_votes']
        unique_together = (('product', 'user'),)

    @models.permalink
    def get_absolute_url(self):
        return ('catalogue:reviews-detail', (), {
            'product_slug': self.product.slug,
            'product_pk': self.product.id,
            'pk': self.id})

    def __unicode__(self):
        return self.title

    def save(self, *args, **kwargs):
        if not self.user and not (self.name and self.email):  
            raise ValidationError("Anonymous review must have a name and an email")
        if not self.title:
            raise ValidationError("Reviews must have a title")
        if self.score is None:
            raise ValidationError("Reviews must have a score")
        super(AbstractProductReview, self).save(*args, **kwargs)

    def has_votes(self):
        return self.total_votes > 0

    def num_up_votes(self):
        """Returns the total up votes"""
        return int((self.total_votes + self.delta_votes) / 2)
    
    def num_down_votes(self):
        """Returns the total down votes"""
        return int((self.total_votes - self.delta_votes) / 2)

    def update_totals(self):
        """Updates total and delta votes"""
        result = self.votes.aggregate(score=Sum('delta'),total_votes=Count('id'))
        self.total_votes = result['total_votes'] or 0
        self.delta_votes = result['score'] or 0
        self.save()
        
    def get_reviewer_name(self):
        if self.user:
            return self.user.username
        else:
            return self.name


class AbstractVote(models.Model):
    """
    Records user ratings as yes/no vote.
    * Only signed-in users can vote.
    * Each user can vote only once.
    """
    review = models.ForeignKey('reviews.ProductReview', related_name='votes')
    user = models.ForeignKey('auth.User', related_name='review_votes')
    UP, DOWN = 1, -1
    VOTE_CHOICES = (
        (UP, _("Up")),
        (DOWN, _("Down"))
    )
    delta = models.SmallIntegerField(choices=VOTE_CHOICES)
    date_created = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True
        ordering = ['-date_created']
        unique_together = (('user', 'review'),)

    def __unicode__(self):
        return u"%s vote for %s" % (self.delta, self.review)

    def save(self, *args, **kwargs):
        u"""
        Validates model and raises error if validation fails
        """
        super(AbstractVote, self).save(*args, **kwargs)
        self.review.update_totals()        



from django.forms import ModelForm, CharField, HiddenInput

from oscar.checkout.forms import ShippingAddressForm as CoreShippingAddressForm
from oscar.services import import_module

address_models = import_module('address.models', ['Country'])
order_models = import_module('order.models', ['ShippingAddress'])


class ShippingAddressForm(CoreShippingAddressForm):
    
    # Overriding form widgets (and labels)
    line4 = CharField(label='County', required=False)
    postcode = CharField(label='Postcode')
    
    class Meta:
        model = order_models.ShippingAddress
        exclude = ('title', 'user', 'notes')
        
    def set_country_queryset(self):
        self.fields['country'].queryset = address_models.Country.objects.filter(is_shipping_country=True)




from logging import Formatter
import re


class PciFormatter(Formatter):
    """
    Strip card numbers out of log messages to avoid leaving sensitive information
    in the logs.
    """
    
    def format(self, record):
        s = Formatter.format(self, record)
        return re.sub(r'\d[ \d-]{15,22}', 'XXXX-XXXX-XXXX-XXXX', s)
        


import operator
from urllib import urlencode

from django.http import HttpResponseRedirect
from django.template import Context, loader, RequestContext
from django.template.response import TemplateResponse
from django.shortcuts import get_object_or_404
from django.core.urlresolvers import reverse
from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.views.generic import ListView, DetailView

from django.template.response import TemplateResponse
from django.contrib import messages
from django.db import transaction
from django.db.models import Q
from django.contrib.comments.views.moderation import delete

from oscar.core.loading import import_module
from oscar.views.generic import PostActionMixin
import_module('order.models', ['Order', 'Line', 'ShippingEvent', 'ShippingEventQuantity', 
                               'ShippingEventType', 'PaymentEvent', 'PaymentEventType', 'OrderNote'], locals())
import_module('order_management.forms', ['SimpleSearch'], locals())


class OrderListView(ListView):
    u"""A list of orders"""
    context_object_name = "orders"
    template_name = 'order_management/browse.html'
    paginate_by = 20

    def get_queryset(self):
        if 'search_query' in self.request.GET and self.request.GET['search_query'].strip():
            q = self.request.GET['search_query'].strip()
            q_list = [Q(number__icontains=q)]
            search_by = self.request.GET.getlist('search_by')
            if search_by:
                if 'billing_address' in search_by:
                    q_list.append(Q(billing_address__search_text__icontains=q))
                if 'shipping_address' in search_by:
                    q_list.append(Q(shipping_address__search_text__icontains=q))
                if 'customer' in search_by:
                    q_list.append(Q(number__icontains=q))
                    q_list.append(Q(user__first_name__icontains=q))
                    q_list.append(Q(user__last_name__icontains=q))
                    q_list.append(Q(user__email__icontains=q))
            return Order._default_manager.filter(reduce(operator.or_, q_list))
        return Order._default_manager.all()
    
    def get_context_data(self, **kwargs):
        context = super(OrderListView, self).get_context_data(**kwargs)
        search_params = self.request.GET.copy()
        if 'page' in search_params:
            del(search_params['page'])
        context['search_params'] = '&' + search_params.urlencode()
        context['order_simple_search_form'] = SimpleSearch(self.request.GET)
        return context
    
    def get(self, request, *args, **kwargs):
        response = super(OrderListView, self).get(request, *args, **kwargs)
        return response
        
        
class OrderDetailView(DetailView, PostActionMixin):
    u"""A detail view of an order"""
    template_name = "order_management/order.html"
    context_object_name = 'order'
    
    def get_object(self):
        u"""Return an order object or a 404"""
        return get_object_or_404(Order, number=self.kwargs['order_number'])
    
    def get_context_data(self, **kwargs):
        context = super(OrderDetailView, self).get_context_data(**kwargs)
        context['shipping_options'] = ShippingEventType._default_manager.all()
        context['payment_options'] = PaymentEventType._default_manager.all()
        return context
      
    def post(self, request, *args, **kwargs):
        order = self.get_object()
        self.response = HttpResponseRedirect(reverse('oscar-order-management-order', kwargs={'order_number': order.number}))
        return super(OrderDetailView, self).post(request, *args, **kwargs)
   
    def do_create_order_event(self, order):
        self.create_shipping_event(order, order.lines.all())
        
    def do_create_line_event(self, order):
        u"""Create an event for an order"""
        line_ids = self.request.POST.getlist('order_line')
        lines = order.lines.in_bulk(line_ids)
        if not len(lines):
            messages.info(self.request, "Please select some lines")
            return
        try:
            if self.request.POST['shipping_event']:
                self.create_shipping_event(order, lines.values())
        except (AttributeError, ValueError), e:
            messages.error(self.request, str(e))    
                
    def create_shipping_event(self, order, lines):
        u"""Create a shipping event for an order"""
        with transaction.commit_on_success():
            event_type = ShippingEventType._default_manager.get(code=self.request.POST['shipping_event'])
            event = ShippingEvent._default_manager.create(order=order, event_type=event_type)
            for line in lines:
                try:
                    event_quantity = int(self.request.POST['order_line_quantity_%d' % line.id])
                except KeyError:
                    event_quantity = line.quantity
                ShippingEventQuantity._default_manager.create(event=event, line=line, 
                                                                  quantity=event_quantity)
            
    def create_payment_event(self, order, lines, type_code):
        u"""Create a payment event for an order"""
        event_type = PaymentEventType._default_manager.get(code=type_code)
        for line in lines.values():
            order_models.PaymentEvent._default_manager.create(order=order, line=line, 
                                                     quantity=line.quantity, event_type=event_type)
            
    def do_add_note(self, order):
        u"""Save a note against an order."""
        if self.request.user.is_authenticated():
            message = self.request.POST['message'].strip()
            if message:
                messages.info(self.request, "Message added")
                OrderNote._default_manager.create(order=order, message=self.request.POST['message'],
                                                         user=self.request.user)
            else:
                messages.info(self.request, "Please enter a message")
    



from django.test.client import Client
from django.test import TestCase
from django.http import HttpRequest

from oscar.apps.customer.models import CommunicationEventType
from oscar.apps.customer.history_helpers import get_recently_viewed_product_ids
from oscar.test.helpers import create_product

class HistoryHelpersTest(TestCase):
    
    def setUp(self):
        self.client = Client()
        self.product = create_product()
    
    def test_viewing_product_creates_cookie(self):
        response = self.client.get(self.product.get_absolute_url())
        self.assertTrue('oscar_recently_viewed_products' in response.cookies)
        
    def test_id_gets_added_to_cookie(self):
        response = self.client.get(self.product.get_absolute_url())
        request = HttpRequest()
        request.COOKIES['oscar_recently_viewed_products'] = response.cookies['oscar_recently_viewed_products'].value
        self.assertTrue(self.product.id in get_recently_viewed_product_ids(request))


class CommunicationTypeTest(TestCase):
    
    keys = ('body', 'html', 'sms', 'subject')
    
    def test_no_templates_returns_empty_string(self):
        et = CommunicationEventType()
        messages = et.get_messages()
        for key in self.keys:
            self.assertEqual('', messages[key])
            
    def test_field_template_render(self):
        et = CommunicationEventType(email_subject_template='Hello {{ name }}')
        ctx = {'name': 'world'}
        messages = et.get_messages(ctx)
        self.assertEqual('Hello world', messages['subject'])




import logging

from django.core.management.base import BaseCommand

from oscar.core.loading import import_module
import_module('analytics.utils', ['ScoreCalculator'], locals())


class Command(BaseCommand):
    
    help = 'Calculate product scores based on analytics data'
    
    def handle(self, *args, **options):
        
        logger = logging.getLogger(__name__)
        calculator = ScoreCalculator(logger)
        calculator.run()

        
        
        
            


from logging import LogRecord

from django.test import TestCase

from oscar.test.decorators import dataProvider
from oscar.core.logging.formatters import PciFormatter


class PciFormatterTests(TestCase):

    def setUp(self):
        self.formatter = PciFormatter() 

    def test_a_basic_string_is_unchanged(self):
        msg = 'some string'
        record = self.create_log_record(msg)
        self.assertEquals(msg, self.formatter.format(record))

    bankcard_strings = lambda: [('here is my bankcard 1000010000000007', 'here is my bankcard XXXX-XXXX-XXXX-XXXX'),
                                ('here is my bankcard 1000-0100-0000-0007', 'here is my bankcard XXXX-XXXX-XXXX-XXXX'),
                                ('here is my bankcard 1000 0100 0000 0007', 'here is my bankcard XXXX-XXXX-XXXX-XXXX'),
                                ('here is my bankcard 10 00 01-00 0-000-0007', 'here is my bankcard XXXX-XXXX-XXXX-XXXX'),
                                ]

    @dataProvider(bankcard_strings)
    def test_a_sensitive_strings_are_filtered(self, sensitive, filtered):
        record = self.create_log_record(sensitive)
        self.assertEquals(filtered, self.formatter.format(record))    
        
    def create_log_record(self, msg):
        return LogRecord(name=None, 
                         level=None, 
                         pathname='', 
                         lineno=0,
                         msg=msg, 
                         args=None, 
                         exc_info=None)
        
        
    



from django.contrib import admin
from django.db.models import get_model

class DiscountOfferAdmin(admin.ModelAdmin):
    list_display = ('name', 'discount_type', 'discount_value', 'num_products')
    list_filter = ('discount_type',)

admin.site.register(get_model('discount', 'discountoffer'), DiscountOfferAdmin)



from django.conf.urls.defaults import *
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from oscar.app import shop

admin.autodiscover()

urlpatterns = patterns('',
    (r'^admin/', include(admin.site.urls)),
    (r'', include(shop.urls)),
)
urlpatterns += staticfiles_urlpatterns()



from django.conf.urls.defaults import patterns, url

from oscar.core.application import Application
from oscar.apps.promotions.views import HomeView, RecordClickView
from oscar.apps.promotions.models import PagePromotion, KeywordPromotion


class PromotionsApplication(Application):
    name = 'promotions'
    
    home_view = HomeView
    record_click_view = RecordClickView

    def get_urls(self):
        urlpatterns = patterns('',
            url(r'page-redirect/(?P<page_promotion_id>\d+)/$', 
                self.record_click_view.as_view(model=PagePromotion), name='page-click'),
            url(r'keyword-redirect/(?P<keyword_promotion_id>\d+)/$', 
                self.record_click_view.as_view(model=KeywordPromotion), name='keyword-click'),
            url(r'^$', self.home_view.as_view(), name='home'),
        )
        return urlpatterns

application = PromotionsApplication()


from oscar.core.loading import import_module
search_forms = import_module('search.forms', ['MultiFacetedSearchForm'])

def search_form(request):
    u"""
    Ensures that the search form is available site wide
    """
    return {'search_form': search_forms.MultiFacetedSearchForm(request.GET)}



from django.template.response import TemplateResponse

from oscar.apps.checkout.views import (PaymentMethodView as CorePaymentMethodView, 
                                  PaymentDetailsView as CorePaymentDetailsView,
                                  OrderPreviewView as CoreOrderPreviewView)
from oscar.apps.payment.forms import BankcardForm, BillingAddressForm
from oscar.apps.shipping.methods import ShippingMethod
from oscar.core.loading import import_module
import_module('payment.models', ['Source', 'SourceType'], locals())
import_module('payment.exceptions', ['TransactionDeclinedException'], locals())
import_module('payment.utils', ['Bankcard'], locals())    
import_module('payment.datacash.utils', ['Gateway', 'Facade'], locals())
import_module('order.models', ['PaymentEvent', 'PaymentEventType', 'PaymentEventQuantity'], locals())
    
    
class PaymentMethodView(CorePaymentMethodView):
    template_file = 'checkout/payment_method.html'
    
    def handle_GET(self):
        return TemplateResponse(self.request, self.template_file, self.context)
    
    def handle_POST(self):
        method = self.request.POST['method_code']
        self.co_data.pay_by(method)
        return self.get_success_response()
    
    
class OrderPreviewView(CoreOrderPreviewView):
    u"""View a preview of the order before submitting."""
    
    def handle_GET(self):
        # Forward straight onto the payment details - no need for preview
        return self.get_success_response()   
        
        
class PaymentDetailsView(CorePaymentDetailsView):
    template_file = 'checkout/payment_details.html'
    
    def handle_GET(self):
        if self.is_cheque_payment():
            self.template_file = 'checkout/payment_details_cheque.html'
        else:
            shipping_addr = self.get_shipping_address()
            card_values = {'name': shipping_addr.name()}
            self.context['bankcard_form'] = BankcardForm(initial=card_values)
            addr_values = {'first_name': shipping_addr.first_name,
                           'last_name': shipping_addr.last_name,}
            self.context['billing_address_form'] = BillingAddressForm(initial=addr_values)
        return TemplateResponse(self.request, self.template_file, self.context)
    
    def handle_POST(self):
        if self.is_cheque_payment():
            return self.submit()
        try:    
            self.bankcard_form = BankcardForm(self.request.POST)
            self.billing_addr_form = BillingAddressForm(self.request.POST)
            if self.bankcard_form.is_valid() and self.billing_addr_form.is_valid():
                return self.submit()
        except TransactionDeclinedException, e:
            self.context['payment_error'] = str(e)
        
        self.context['bankcard_form'] = self.bankcard_form
        self.context['billing_address_form'] = self.billing_addr_form
        return TemplateResponse(self.request, self.template_file, self.context)

    def handle_payment(self, order_number, total):
        if self.is_cheque_payment():
            self.handle_cheque_payment(total)
        else:    
            self.handle_bankcard_payment(order_number, total)

    def is_cheque_payment(self):
        payment_method = self.co_data.payment_method()
        return payment_method == 'cheque'

    def handle_cheque_payment(self, total):
        # Nothing to do except create a payment source
        type,_ = SourceType.objects.get_or_create(name="Cheque")
        source = Source(type=type, allocation=total)
        self.payment_sources.append(source)

    def handle_bankcard_payment(self, order_number, total):
        # Handle payment problems with an exception
        # Make payment submission - handle response from DC
        # - could be an iframe open
        # - could be failure
        # - could be redirect
        
        # Create bankcard object
        bankcard = self.bankcard_form.get_bankcard_obj()
        
        # Handle new card submission 
        dc_facade = Facade()
        reference = dc_facade.debit(order_number, total, bankcard, self.basket)
        
        # Create payment source (but don't save just yet)
        type,_ = SourceType.objects.get_or_create(name='DataCash', code='datacash')
        source = Source(type=type,
               allocation=total,
               amount_debited=total,
               reference=reference)
        self.payment_sources.append(source)

    def place_order(self, basket, order_number, total_incl_tax, total_excl_tax):
        order = super(PaymentDetailsView, self).place_order(basket, order_number, total_incl_tax, total_excl_tax)
        if self.is_cheque_payment():
            order.status = "Awaiting cheque"
            order.save()
        return order
    
    def create_billing_address(self):
        if not hasattr(self, 'billing_addr_form'):
            return None
        return self.billing_addr_form.save()
        
    def save_payment_events(self, order):
        event_type,_ = PaymentEventType.objects.get_or_create(code="paid-for")
        event = PaymentEvent.objects.create(order=order, event_type=event_type)
        for line in order.lines.all():
            line_qty = PaymentEventQuantity.objects.create(event=event, 
                                                           line=line,
                                                           quantity=line.quantity)



from django.forms import fields
from django.core.validators import URL_VALIDATOR_USER_AGENT

from oscar.core import validators

class ExtendedURLField(fields.URLField):
    """
    Custom field similar to URLField type field, however also accepting 
    and validating local relative URLs, ie. '/product/'
    """

    def __init__(self, max_length=None, min_length=None, verify_exists=False,
            validator_user_agent=URL_VALIDATOR_USER_AGENT, *args, **kwargs):
        # intentionally skip one step when calling super()
        super(fields.URLField, self).__init__(max_length, min_length, *args,
                                       **kwargs)
        validator = validators.ExtendedURLValidator(verify_exists=verify_exists,
                                                    validator_user_agent=validator_user_agent)
        self.validators.append(validator)

    def to_python(self, value):
        # We need to avoid having 'http' inserted at the start of
        # every value
        if value and value.startswith('/'):
            return value
        return super(ExtendedURLField, self).to_python(value)




import urlparse

from django.shortcuts import get_object_or_404
from django.views.generic import TemplateView, ListView, DetailView, CreateView, UpdateView, DeleteView
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.contrib import messages
from django.utils.translation import ugettext as _
from django.contrib.auth import authenticate, login as auth_login
from django.contrib.sites.models import get_current_site
from django.conf import settings
from django.db.models import get_model

from oscar.apps.address.forms import UserAddressForm
from oscar.views.generic import PostActionMixin
from oscar.apps.customer.forms import EmailAuthenticationForm, EmailUserCreationForm
from oscar.core.loading import import_module
import_module('customer.utils', ['Dispatcher'], locals())

order_model = get_model('order', 'Order')
order_line_model = get_model('order', 'Line')
basket_model = get_model('basket', 'Basket')
user_address_model = get_model('address', 'UserAddress')
email_model = get_model('customer', 'email')
communicationtype_model = get_model('customer', 'communicationeventtype')


class AccountSummaryView(ListView):
    """Customer order history"""
    context_object_name = "orders"
    template_name = 'customer/profile.html'
    paginate_by = 20
    model = order_model

    def get_queryset(self):
        """Return a customer's orders"""
        return self.model._default_manager.filter(user=self.request.user)[0:5]
    
    
class AccountAuthView(TemplateView):
    template_name = 'customer/login_registration.html'
    redirect_field_name = 'next'
    login_prefix = 'login'
    registration_prefix = 'registration'
    communication_type_code = 'REGISTRATION'
    
    def get_logged_in_redirect(self):
        return reverse('customer:summary')
    
    def get_context_data(self, *args, **kwargs):
        context = super(AccountAuthView, self).get_context_data(*args, **kwargs)
        redirect_to = self.request.REQUEST.get(self.redirect_field_name, '')
        context[self.redirect_field_name] = redirect_to
        context['login_form'] = EmailAuthenticationForm(prefix=self.login_prefix)
        context['registration_form'] = EmailUserCreationForm(prefix=self.registration_prefix)        
        return context
    
    def check_redirect(self, context):
        redirect_to = context.get(self.redirect_field_name)
        
        netloc = urlparse.urlparse(redirect_to)[1]
        if not redirect_to:
            redirect_to = settings.LOGIN_REDIRECT_URL
        elif netloc and netloc != self.request.get_host():
            redirect_to = settings.LOGIN_REDIRECT_URL
        return redirect_to
    
    def send_registration_email(self, user):
        code = self.communication_type_code
        ctx = {'user': user,
               'site': get_current_site(self.request)}
        try:
            event_type = communicationtype_model.objects.get(code=code)
        except communicationtype_model.DoesNotExist:
            # No event in database, attempt to find templates for this type
            messages = communicationtype_model.objects.get_and_render(code, ctx)
        else:
            # Create order event
            messages = event_type.get_messages(ctx)

        if messages and messages['body']:      
            dispatcher = Dispatcher()
            dispatcher.dispatch_user_messages(user, messages)
    
    def get(self, request, *args, **kwargs):
        context = self.get_context_data(*args, **kwargs)
        
        if request.user.is_authenticated():
            return HttpResponseRedirect(self.get_logged_in_redirect())

        self.request.session.set_test_cookie()
        return self.render_to_response(context)
    
    def post(self, request, *args, **kwargs):
        context = self.get_context_data(*args, **kwargs)
        redirect_to = self.check_redirect(context)
        
        if u'login_submit' in self.request.POST:
            login_form = EmailAuthenticationForm(prefix=self.login_prefix, data=request.POST)            
            if login_form.is_valid():
                auth_login(request, login_form.get_user())
                if request.session.test_cookie_worked():
                    request.session.delete_test_cookie()
                return HttpResponseRedirect(redirect_to)
            context['login_form'] = login_form

        if u'registration_submit' in self.request.POST:
            registration_form = EmailUserCreationForm(prefix=self.registration_prefix, data=request.POST)
            context['registration_form'] = registration_form
            if registration_form.is_valid():
                user = registration_form.save()
                
                if getattr(settings, 'OSCAR_SEND_REGISTRATION_EMAIL', True):
                    self.send_registration_email(user)
                
                user = authenticate(username=user.email, password=registration_form.cleaned_data['password1'])
                auth_login(self.request, user)
                if self.request.session.test_cookie_worked():
                    self.request.session.delete_test_cookie()                
                return HttpResponseRedirect(redirect_to)
        
        self.request.session.set_test_cookie()
        return self.render_to_response(context)

    
class EmailHistoryView(ListView):
    """Customer email history"""
    context_object_name = "emails"
    template_name = 'customer/email-history.html'
    paginate_by = 20

    def get_queryset(self):
        """Return a customer's orders"""
        return email_model._default_manager.filter(user=self.request.user)


class EmailDetailView(DetailView):
    """Customer order details"""
    template_name = "customer/email.html"
    context_object_name = 'email'
    
    def get_object(self):
        """Return an order object or 404"""
        return get_object_or_404(email_model, user=self.request.user, id=self.kwargs['email_id'])


class OrderHistoryView(ListView):
    """Customer order history"""
    context_object_name = "orders"
    template_name = 'customer/order-history.html'
    paginate_by = 20
    model = order_model

    def get_queryset(self):
        """Return a customer's orders"""
        return self.model._default_manager.filter(user=self.request.user)


class OrderDetailView(DetailView):
    """Customer order details"""
    model = order_model
    
    def get_template_names(self):
        return ["customer/order.html"]    

    def get_object(self):
        return get_object_or_404(self.model, user=self.request.user, number=self.kwargs['order_number'])


class OrderLineView(DetailView, PostActionMixin):
    """Customer order line"""
    
    def get_object(self):
        """Return an order object or 404"""
        order = get_object_or_404(order_model, user=self.request.user, number=self.kwargs['order_number'])
        return order.lines.get(id=self.kwargs['line_id'])
    
    def do_reorder(self, line):
        if not line.product:
            messages.info(self.request, _("This product is no longer available for re-order"))
            return
        
        # We need to pass response to the get_or_create... method
        # as a new basket might need to be created
        self.response = HttpResponseRedirect(reverse('basket:summary'))
        basket = self.request.basket
        
        # Convert line attributes into basket options
        options = []
        for attribute in line.attributes.all():
            if attribute.option:
                options.append({'option': attribute.option, 'value': attribute.value})
        basket.add_product(line.product, 1, options)
        messages.info(self.request, "Line reordered")  


class AddressListView(ListView):
    """Customer address book"""
    context_object_name = "addresses"
    template_name = 'customer/address-book.html'
    paginate_by = 40
        
    def get_queryset(self):
        """Return a customer's addresses"""
        return user_address_model._default_manager.filter(user=self.request.user)


class AddressCreateView(CreateView):
    form_class = UserAddressForm
    mode = user_address_model
    
    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        return HttpResponseRedirect(self.get_success_url())

    def get_template_names(self):
        return ["customer/address-create.html"]

    def get_success_url(self):
        return reverse('customer:address-list')


class AddressUpdateView(UpdateView):
    form_class = UserAddressForm
    model = user_address_model
    
    def get_queryset(self):
        """Return a customer's addresses"""
        return user_address_model._default_manager.filter(user=self.request.user)    

    def get_template_names(self):
        return ["customer/address-form.html"]
    
    def get_success_url(self):
        return reverse('customer:address-detail', kwargs={'pk': self.get_object().pk })


class AddressDeleteView(DeleteView):
    model = user_address_model

    def get_queryset(self):
        """Return a customer's addresses"""
        return user_address_model._default_manager.filter(user=self.request.user)        

    def get_success_url(self):
        return reverse('customer:address-list')    
    
    def get_template_names(self):
        return ["customer/address-delete.html"]




class PostActionMixin(object):
    """
    Simple mixin to forward POST request that contain a key 'action'
    onto a method of form "do_{action}".  This makes some DetailViews
    easier to write.
    """
    def post(self, request, *args, **kwargs):
        if 'action' in self.request.POST:
            model = self.get_object()
            getattr(self, "do_%s" % self.request.POST['action'].lower())(model)
            return self.response
    


from decimal import Decimal
import logging

from django.conf import settings
from django.http import HttpResponse, Http404, HttpResponseRedirect, HttpResponseBadRequest
from django.template import RequestContext
from django.shortcuts import get_object_or_404
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.contrib import messages
from django.core.urlresolvers import resolve
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext as _
from django.template.response import TemplateResponse
from django.core.mail import EmailMessage
from django.views.generic import DetailView, TemplateView, FormView, \
                                 DeleteView, UpdateView, CreateView

from oscar.apps.shipping.methods import FreeShipping
from oscar.core.loading import import_module
import_module('checkout.forms', ['ShippingAddressForm'], locals())
import_module('checkout.calculators', ['OrderTotalCalculator'], locals())
import_module('checkout.utils', ['CheckoutSessionData'], locals())
import_module('checkout.signals', ['pre_payment', 'post_payment'], locals())
import_module('order.models', ['Order', 'ShippingAddress',
                               'CommunicationEvent'], locals())
import_module('order.utils', ['OrderNumberGenerator', 'OrderCreator'], locals())
import_module('address.models', ['UserAddress'], locals())
import_module('address.forms', ['UserAddressForm'], locals())
import_module('shipping.repository', ['Repository'], locals())
import_module('customer.models', ['Email', 'CommunicationEventType'], locals())
import_module('customer.views', ['AccountAuthView'], locals())
import_module('customer.utils', ['Dispatcher'], locals())
import_module('payment.exceptions', ['RedirectRequired', 'UnableToTakePayment', 
                                     'PaymentError'], locals())
import_module('basket.models', ['Basket'], locals())

# Standard logger for checkout events
logger = logging.getLogger('oscar.checkout')


class IndexView(AccountAuthView):
    """
    First page of the checkout.  If the user is signed in then we forward
    straight onto the next step.  Otherwise, we provide options to login, register and
    (if the option is enabled) proceed anonymously.
    """
    template_name = 'checkout/gateway.html'
    
    def get_logged_in_redirect(self):
        return reverse('checkout:shipping-address')


class CheckoutSessionMixin(object):
    """
    Mixin to provide common functionality shared between checkout views.
    """   

    def dispatch(self, request, *args, **kwargs):
        self.checkout_session = CheckoutSessionData(request)
        return super(CheckoutSessionMixin, self).dispatch(request, *args, **kwargs)

    def get_shipping_address(self):
        """
        Return the current shipping address for this checkout session.
        
        This could either be a ShippingAddress model which has been
        pre-populated (not saved), or a UserAddress model which will 
        need converting into a ShippingAddress model at submission
        """
        addr_data = self.checkout_session.new_shipping_address_fields()
        if addr_data:
            # Load address data into a blank address model
            return ShippingAddress(**addr_data)
        addr_id = self.checkout_session.user_address_id()
        if addr_id:
            try:
                return UserAddress._default_manager.get(pk=addr_id)
            except UserAddress.DoesNotExist:
                # This can happen if you reset all your tables and you still have
                # session data that refers to addresses that no longer exist
                pass
        return None
        
    def get_shipping_method(self, basket=None):
        method = self.checkout_session.shipping_method()
        if method:
            if not basket:
                basket = self.request.basket
            method.set_basket(basket)
        else:
            # We default to using free shipping
            method = FreeShipping()
        return method
    
    def get_order_totals(self, basket=None, shipping_method=None, **kwargs):
        """
        Returns the total for the order with and without tax (as a tuple)
        """
        calc = OrderTotalCalculator(self.request)
        if not basket:
            basket = self.request.basket
        if not shipping_method:
            shipping_method = self.get_shipping_method(basket)
        total_incl_tax = calc.order_total_incl_tax(basket, shipping_method, **kwargs)
        total_excl_tax = calc.order_total_excl_tax(basket, shipping_method, **kwargs)
        return total_incl_tax, total_excl_tax
    
    def get_context_data(self, **kwargs):
        """
        Assign common template variables to the context.
        """
        ctx = super(CheckoutSessionMixin, self).get_context_data(**kwargs)
        ctx['shipping_address'] = self.get_shipping_address()
        
        method = self.get_shipping_method()
        if method:
            ctx['shipping_method'] = method
            ctx['shipping_total_excl_tax'] = method.basket_charge_excl_tax()
            ctx['shipping_total_incl_tax'] = method.basket_charge_incl_tax()
            
        ctx['order_total_incl_tax'], ctx['order_total_excl_tax'] = self.get_order_totals()
        
        return ctx


# ================
# SHIPPING ADDRESS
# ================


class ShippingAddressView(CheckoutSessionMixin, FormView):
    """
    Determine the shipping address for the order.
    
    The default behaviour is to display a list of addresses from the users's
    address book, from which the user can choose one to be their shipping address.
    They can add/edit/delete these USER addresses.  This address will be
    automatically converted into a SHIPPING address when the user checks out.
    
    Alternatively, the user can enter a SHIPPING address directly which will be
    saved in the session and saved as a model when the order is sucessfully submitted.
    """
    
    template_name = 'checkout/shipping_address.html'
    form_class = ShippingAddressForm
    
    def get_initial(self):
        return self.checkout_session.new_shipping_address_fields()
    
    def get_context_data(self, **kwargs):
        kwargs = super(ShippingAddressView, self).get_context_data(**kwargs)
        if self.request.user.is_authenticated():
            # Look up address book data
            kwargs['addresses'] = self.get_available_addresses()
        return kwargs
    
    def get_available_addresses(self):
        return UserAddress._default_manager.filter(user=self.request.user)
    
    def post(self, request, *args, **kwargs):
        # Check if a shipping address was selected directly (eg no form was filled in)
        if self.request.user.is_authenticated and 'address_id' in self.request.POST:
            address = UserAddress._default_manager.get(pk=self.request.POST['address_id'])
            if 'action' in self.request.POST and self.request.POST['action'] == 'ship_to':
                # User has selected a previous address to ship to
                self.checkout_session.ship_to_user_address(address)
                return HttpResponseRedirect(self.get_success_url())
            elif 'action' in self.request.POST and self.request.POST['action'] == 'delete':
                address.delete()
                messages.info(self.request, "Address deleted from your address book")
                return HttpResponseRedirect(reverse('checkout:shipping-method'))
            else:
                return HttpResponseBadRequest()
        else:
            return super(ShippingAddressView, self).post(request, *args, **kwargs)
    
    def form_valid(self, form):
        self.checkout_session.ship_to_new_address(form.clean())
        return super(ShippingAddressView, self).form_valid(form)
    
    def get_success_url(self):
        return reverse('checkout:shipping-method')
    

class UserAddressCreateView(CheckoutSessionMixin, CreateView):
    """
    Add a USER address to the user's addressbook.

    This is not the same as creating a SHIPPING Address, although if used for the order,
    it will be converted into a shipping address at submission-time.
    """
    template_name = 'checkout/user_address_form.html'
    form_class = UserAddressForm

    def get_context_data(self, **kwargs):
        kwargs = super(UserAddressCreateView, self).get_context_data(**kwargs)
        kwargs['form_url'] = reverse('checkout:user-address-create')
        return kwargs
    
    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.user = self.request.user
        self.object.save()
        return self.get_success_response()

    def get_success_response(self):
        messages.info(self.request, _("Address saved"))
        # We redirect back to the shipping address page
        return HttpResponseRedirect(reverse('checkout:shipping-address'))
    
    
class UserAddressUpdateView(CheckoutSessionMixin, UpdateView):
    """
    Update a user address
    """
    template_name = 'checkout/user_address_form.html'
    form_class = UserAddressForm
    
    def get_queryset(self):
        return UserAddress._default_manager.filter(user=self.request.user)

    def get_context_data(self, **kwargs):
        kwargs = super(UserAddressUpdateView, self).get_context_data(**kwargs)
        kwargs['form_url'] = reverse('checkout:user-address-update', args=(str(kwargs['object'].id),))
        return kwargs

    def get_success_url(self):
        messages.info(self.request, _("Address saved"))
        return reverse('checkout:shipping-address')
    
    
class UserAddressDeleteView(CheckoutSessionMixin, DeleteView):
    """
    Delete an address from a user's addressbook.
    """

    def get_queryset(self):
        return UserAddress._default_manager.filter(user=self.request.user)
    
    def get_success_url(self):
        messages.info(self.request, _("Address deleted"))
        return reverse('checkout:shipping-address')
    

# ===============    
# Shipping method
# ===============    
    

class ShippingMethodView(CheckoutSessionMixin, TemplateView):
    """
    View for allowing a user to choose a shipping method.
    
    Shipping methods are largely domain-specific and so this view
    will commonly need to be subclassed and customised.
    
    The default behaviour is to load all the available shipping methods
    using the shipping Repository.  If there is only 1, then it is 
    automatically selected.  Otherwise, a page is rendered where
    the user can choose the appropriate one.
    """
    template_name = 'checkout/shipping_methods.html';
    
    def get(self, request, *args, **kwargs):
        # Save shipping methods as instance var as we need them both here
        # and when setting the context vars.
        self._methods = self.get_available_shipping_methods()
        if len(self._methods) == 1:
            # Only one shipping method - set this and redirect onto the next step
            self.checkout_session.use_shipping_method(self._methods[0].code)
            return self.get_success_response()
        return super(ShippingMethodView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        kwargs = super(ShippingMethodView, self).get_context_data(**kwargs)
        kwargs['methods'] = self._methods
        return kwargs

    def get_available_shipping_methods(self):
        """
        Returns all applicable shipping method objects
        for a given basket.
        """ 
        repo = Repository()
        # Shipping methods can depend on the user, the contents of the basket
        # and the shipping address.  I haven't come across a scenario that doesn't
        # fit this system.
        return repo.get_shipping_methods(self.request.user, self.request.basket, 
                                         self.get_shipping_address())
    
    def post(self, request, *args, **kwargs):
        method_code = request.POST['method_code']
        # Save the code for the chosen shipping method in the session
        # and continue to the next step.
        self.checkout_session.use_shipping_method(method_code)
        return self.get_success_response()
        
    def get_success_response(self):
        return HttpResponseRedirect(reverse('checkout:payment-method'))


class PaymentMethodView(CheckoutSessionMixin, TemplateView):
    """
    View for a user to choose which payment method(s) they want to use.
    
    This would include setting allocations if payment is to be split
    between multiple sources.
    """
    
    def get(self, request, *args, **kwargs):
        return self.get_success_response()
    
    def get_success_response(self):
        return HttpResponseRedirect(reverse('checkout:preview'))


class OrderPreviewView(CheckoutSessionMixin, TemplateView):
    """
    View a preview of the order before submitting.
    """
    template_name = 'checkout/preview.html'
    
    def get_success_response(self):
        return HttpResponseRedirect(reverse('checkout:payment-details'))


class OrderPlacementMixin(CheckoutSessionMixin):
    """
    Mixin for providing functionality for placing orders.
    """
    # Any payment sources should be added to this list as part of the
    # _handle_payment method.  If the order is placed successfully, then
    # they will be persisted.
    _payment_sources = None
    communication_type_code = 'ORDER_PLACED' 
    
    def handle_order_placement(self, order_number, basket, total_incl_tax, total_excl_tax, **kwargs): 
        """
        Place the order into the database and return the appropriate HTTP response
        
        We deliberately pass the basket in here as the one tied to the request
        isn't necessarily the correct one to use in placing the order.  This can
        happen when a basket gets frozen.
        """   
        # Write out all order and payment models
        order = self.place_order(order_number, basket, total_incl_tax, total_excl_tax, **kwargs)
        basket.set_as_submitted()
        return self.handle_successful_order(order)
        
    def add_payment_source(self, source):
        if self._payment_sources is None:
            self._payment_sources = []
        self._payment_sources.append(source)  
        
    def handle_successful_order(self, order):  
        """
        Handle the various steps required after an order has been successfully placed.
        """  
        # Send confirmation message (normally an email)
        self.send_confirmation_message(order)
        
        # Flush all session data
        self.checkout_session.flush()
        
        # Save order id in session so thank-you page can load it
        self.request.session['checkout_order_id'] = order.id
        return HttpResponseRedirect(reverse('checkout:thank-you'))
    
    def place_order(self, order_number, basket, total_incl_tax, total_excl_tax, **kwargs):
        """
        Writes the order out to the DB including the payment models
        """
        shipping_address = self.create_shipping_address()
        shipping_method = self.get_shipping_method(basket)
        billing_address = self.create_billing_address(shipping_address)
        if 'status' not in kwargs:
            status = self.get_initial_order_status(basket)
        else:
            status = kwargs.pop('status')
        order = OrderCreator().place_order(self.request.user, 
                                         basket, 
                                         shipping_address, 
                                         shipping_method, 
                                         billing_address,
                                         total_incl_tax,
                                         total_excl_tax,
                                         order_number,
                                         status,
                                         **kwargs)
        self.save_payment_details(order)
        return order
    
    def create_shipping_address(self):
        """
        Create and returns the shipping address for the current order.
        
        If the shipping address was entered manually, then we simply
        write out a ShippingAddress model with the appropriate form data.  If
        the user is authenticated, then we create a UserAddress from this data
        too so it can be re-used in the future. 
        
        If the shipping address was selected from the user's address book,
        then we convert the UserAddress to a ShippingAddress.
        """
        addr_data = self.checkout_session.new_shipping_address_fields()
        addr_id = self.checkout_session.user_address_id()
        if addr_data:
            addr = self.create_shipping_address_from_form_fields(addr_data)
            self.create_user_address(addr_data)
        elif addr_id:
            addr = self.create_shipping_address_from_user_address(addr_id)
        else:
            raise AttributeError("No shipping address data found")
        return addr
    
    def create_shipping_address_from_form_fields(self, addr_data):
        """Creates a shipping address model from the saved form fields"""
        shipping_addr = ShippingAddress(**addr_data)
        shipping_addr.save() 
        return shipping_addr
    
    def create_user_address(self, addr_data):
        """
        For signed-in users, we create a user address model which will go 
        into their address book.
        """
        if self.request.user.is_authenticated():
            addr_data['user_id'] = self.request.user.id
            user_addr = UserAddress(**addr_data)
            # Check that this address isn't already in the db as we don't want
            # to fill up the customer address book with duplicate addresses
            try:
                UserAddress._default_manager.get(hash=user_addr.generate_hash())
            except ObjectDoesNotExist:
                user_addr.save()
    
    def create_shipping_address_from_user_address(self, addr_id):
        """Creates a shipping address from a user address"""
        address = UserAddress._default_manager.get(pk=addr_id)
        # Increment the number of orders to help determine popularity of orders 
        address.num_orders += 1
        address.save()
        
        shipping_addr = ShippingAddress()
        address.populate_alternative_model(shipping_addr)
        shipping_addr.save()
        return shipping_addr
    
    def create_billing_address(self, shipping_address=None):
        """
        Saves any relevant billing data (eg a billing address).
        """
        return None

    def save_payment_details(self, order):
        """
        Saves all payment-related details. This could include a billing 
        address, payment sources and any order payment events.
        """
        self.save_payment_events(order)
        self.save_payment_sources(order)
    
    def save_payment_events(self, order):
        """
        Saves any relevant payment events for this order
        """
        pass

    def save_payment_sources(self, order):
        """
        Saves any payment sources used in this order.
        
        When the payment sources are created, the order model does not exist and 
        so they need to have it set before saving.
        """
        if not self._payment_sources:
            return
        for source in self._payment_sources:
            source.order = order
            source.save()
    
    def get_initial_order_status(self, basket):
        return None
        
    def get_submitted_basket(self):
        basket_id = self.checkout_session.get_submitted_basket_id()
        return Basket._default_manager.get(pk=basket_id)
    
    def restore_frozen_basket(self):
        """
        Restores a frozen basket as the sole OPEN basket.  Note that this also merges
        in any new products that have been added to a basket that has been created while payment.
        """
        try:
            fzn_basket = self.get_submitted_basket()
        except Basket.DoesNotExist:
            # Strange place.  The previous basket stored in the session does
            # not exist.
            pass
        else:
            fzn_basket.thaw()
            if self.request.basket.id != fzn_basket.id:
                fzn_basket.merge(self.request.basket)
                self.request.basket = fzn_basket

    def send_confirmation_message(self, order):
        code = self.communication_type_code
        ctx = {'order': order}
        try:
            event_type = CommunicationEventType.objects.get(code=code)
        except CommunicationEventType.DoesNotExist:
            # No event in database, attempt to find templates for this type
            messages = CommunicationEventType.objects.get_and_render(code, ctx)
            event_type = None
        else:
            # Create order event
            CommunicationEvent._default_manager.create(order=order, type=event_type)
            messages = event_type.get_messages(ctx)

        if messages and messages['body']:      
            logger.info("Order #%s - sending %s messages", order.number, code)  
            dispatcher = Dispatcher(logger)
            dispatcher.dispatch_order_messages(order, messages, event_type)
        else:
            logger.warning("Order #%s - no %s communication event type", order.number, code)


class PaymentDetailsView(OrderPlacementMixin, TemplateView):
    """
    For taking the details of payment and creating the order
    
    The class is deliberately split into fine-grained methods, responsible for only one
    thing.  This is to make it easier to subclass and override just one component of
    functionality.
    
    Almost all projects will need to subclass and customise this class.
    """
    
    def post(self, request, *args, **kwargs):
        """
        This method is designed to be overridden by subclasses which will
        validate the forms from the payment details page.  If the forms are valid
        then the method can call submit()."""
        return self.submit(request.basket, **kwargs)
    
    def submit(self, basket, **kwargs):
        """
        Submit a basket for order placement.
        
        The process runs as follows:
         * Generate an order number
         * Freeze the basket so it cannot be modified any more.
         * Attempt to take payment for the order
           - If payment is successful, place the order
           - If a redirect is required (eg PayPal, 3DSecure), redirect
           - If payment is unsuccessful, show an appropriate error message
        """
        # We generate the order number first as this will be used
        # in payment requests (ie before the order model has been 
        # created).  We also save it in the session for multi-stage
        # checkouts (eg where we redirect to a 3rd party site and place
        # the order on a different request).
        order_number = self.generate_order_number(basket)
        logger.info("Order #%s: beginning submission process for basket %d", order_number, basket.id)
        
        # We freeze the basket to prevent it being modified once the payment
        # process has started.  If your payment fails, then the basket will
        # need to be "unfrozen".  We also store the basket ID in the session
        # so the it can be retrieved by multistage checkout processes.
        basket.freeze()
        self.checkout_session.set_submitted_basket(basket)
        
        # Handle payment.  Any payment problems should be handled by the 
        # handle_payment method raise an exception, which should be caught
        # within handle_POST and the appropriate forms redisplayed.
        try:
            pre_payment.send_robust(sender=self, view=self)
            total_incl_tax, total_excl_tax = self.get_order_totals(basket)
            self.handle_payment(order_number, total_incl_tax, **kwargs)
            post_payment.send_robust(sender=self, view=self)
        except RedirectRequired, e:
            # Redirect required (eg PayPal, 3DS)
            logger.info("Order #%s: redirecting to %s", order_number, e.url)
            return HttpResponseRedirect(e.url)
        except UnableToTakePayment, e:
            # Something went wrong with payment, need to show
            # error to the user.  This type of exception is supposed
            # to set a friendly error message.
            logger.warning("Order #%s: unable to take payment (%s) - restoring basket", order_number, e)
            self.restore_frozen_basket()
            return self.render_to_response(self.get_context_data(error=str(e)))
        except PaymentError, e:
            # Something went wrong which wasn't anticipated.
            logger.error("Order #%s: payment error (%s)", order_number, e)
            self.restore_frozen_basket()
            return self.render_to_response(self.get_context_data(error="A problem occurred processing payment."))
        else:
            # If all is ok with payment, place order
            logger.info("Order #%s: payment successful, placing order", order_number)
            return self.handle_order_placement(order_number, basket, total_incl_tax, total_excl_tax, **kwargs)
    
    def generate_order_number(self, basket):
        generator = OrderNumberGenerator()
        order_number = generator.order_number(basket)
        self.checkout_session.set_order_number(order_number)
        return order_number
    
    def handle_payment(self, order_number, total, **kwargs):
        """
        Handle any payment processing.  
        
        This method is designed to be overridden within your project.  The
        default is to do nothing.
        """
        pass
        

class ThankYouView(DetailView):
    """
    Displays the 'thank you' page which summarises the order just submitted.
    """
    template_name = 'checkout/thank_you.html'
    context_object_name = 'order'
    
    def get_object(self):
        # We allow superusers to force an order thankyou page for testing
        order = None
        if self.request.user.is_superuser:
            if 'order_number' in self.request.GET:
                order = Order._default_manager.get(number=self.request.GET['order_number'])
            elif 'order_id' in self.request.GET:
                order = Order._default_manager.get(id=self.request.GET['orderid'])
        
        if not order:
            if 'checkout_order_id' in self.request.session:
                order = Order._default_manager.get(pk=self.request.session['checkout_order_id'])
            else:
                raise Http404(_("No order found"))
        
        return order



from decimal import Decimal
from itertools import chain
import logging

from oscar.core.loading import import_module
import_module('offer.models', ['ConditionalOffer'], locals())

logger = logging.getLogger('oscar.offers')


class OfferApplicationError(Exception):
    pass

# This needs hooking into the offer application system.
class Discount(object):
    
    def __init__(self, offer):
        self.offer = offer
        self.discount = Decimal('0.00')
        self.frequency = 0
        
    def discount(self, discount):
        self.discount += discount
        self.frequency += 1
        
    def is_voucher_discount(self):
        return bool(self.offer.get_voucher())

    def get_voucher(self):
        return self.offer.get_voucher()


class Applicator(object):
    """
    For applying offers to a basket.
    """
    max_applications = 50
    
    def apply(self, request, basket):
        """
        Apply all relevant offers to the given basket.  The request and user is passed 
        too as sometimes the available offers are dependent on the user (eg session-based
        offers).
        """
        offers = self.get_offers(request, basket) 
        logger.debug("Found %d offers to apply to basket %d", len(offers), basket.id)
        discounts = self.get_basket_discounts(basket, offers)
        # Store this list of discounts with the basket so it can be 
        # rendered in templates
        basket.set_discounts(list(discounts.values()))
        
    def get_basket_discounts(self, basket, offers):    
        discounts = {}
        for offer in offers:
            # For each offer, we keep trying to apply it until the
            # discount is 0
            applications = 0
            while True:
                discount = offer.apply_benefit(basket)
                applications += 1
                logger.debug("Found discount %.2f for basket %d from offer %d", discount, basket.id, offer.id)
                if discount > 0:
                    if offer.id not in discounts:
                        discounts[offer.id] = {'name': offer.name,
                                               'offer': offer,
                                               'voucher': offer.get_voucher(),
                                               'freq': 0,
                                               'discount': Decimal('0.00')} 
                    discounts[offer.id]['discount'] += discount
                    discounts[offer.id]['freq'] += 1
                else:
                    break
                if applications > self.max_applications:
                    logger.error("Exceeded %d applications for offer %d on basket %d", self.max_applications, offer.id, basket.id)
                    raise OfferApplicationError("Exceeded %d applications for offer %d on basket %d" % (self.max_applications, offer.id, basket.id))
        
        logger.debug("Finished applying offers to basket %d", basket.id)
        return discounts
    
    def get_offers(self, request, basket):
        """
        Return all offers to apply to the basket.
        
        This method should be subclassed and extended to provide more sophisticated
        behaviour.  For instance, you could load extra offers based on the session or
        the user type.
        """
        site_offers = self.get_site_offers()
        basket_offers = self.get_basket_offers(basket, request.user)
        user_offers = self.get_user_offers(request.user)
        session_offers = self.get_session_offers(request)
        
        return list(chain(session_offers, basket_offers, user_offers, site_offers))
        
    def get_site_offers(self):
        """
        Return site offers that are available to all users
        """
        return ConditionalOffer.active.filter(offer_type="Site")
    
    def get_basket_offers(self, basket, user):
        """
        Return basket-linked offers such as those associated with a voucher code"""
        offers = []
        if not basket.id:
            return offers
        
        for voucher in basket.vouchers.all():
            if voucher.is_active() and voucher.is_available_to_user(user):
                basket_offers = voucher.offers.all()
                for offer in basket_offers:
                    offer.set_voucher(voucher)
                offers = list(chain(offers, basket_offers))
        return offers
    
    def get_user_offers(self, user):
        """
        Returns offers linked to this particular user.  
        
        Eg: student users might get 25% off
        """
        return []
    
    def get_session_offers(self, request):
        """
        Returns temporary offers linked to the current session.
        
        Eg: visitors coming from an affiliate site get a 10% discount
        """
        return []
        



from django.dispatch import receiver
from django.db.models.signals import m2m_changed, post_save

from oscar.core.loading import import_module
from oscar.apps.basket.abstract_models import AbstractBasket
import_module('voucher.models', ['Voucher'], locals())
import_module('order.models', ['OrderDiscount'], locals())

@receiver(m2m_changed)
def receive_basket_voucher_change(sender, **kwargs):
    if (kwargs['model'] == Voucher and kwargs['action'] == 'post_add' and
        isinstance(kwargs['instance'], AbstractBasket) and kwargs['pk_set']):
        voucher_id = list(kwargs['pk_set'])[0]
        voucher = Voucher._default_manager.get(pk=voucher_id)
        voucher.num_basket_additions += 1
        voucher.save()

@receiver(post_save, sender=OrderDiscount)        
def receive_order_discount_save(sender, instance, **kwargs):
    # Record the amount of discount against the appropriate offers
    # and vouchers
    discount = instance
    if discount.voucher:
        discount.voucher.total_discount += discount.amount
        discount.voucher.save()
    discount.offer.total_discount += discount.amount
    discount.offer.save()
    
    
        
    



from oscar.apps.discount.abstract_models import (AbstractDiscountOffer, PERCENTAGE_DISCOUNT, 
                                            ABSOLUTE_DISCOUNT, FINAL_PRICE)

class DiscountOffer(AbstractDiscountOffer):
    pass




import datetime
from decimal import Decimal as D

from django.utils.translation import ugettext_lazy as _


class DefaultWrapper(object):
    """
    Default stockrecord wrapper
    """
    
    def is_available_to_buy(self, stockrecord):
        return stockrecord.num_in_stock > 0
    
    def availability(self, stockrecord):
        if stockrecord.num_in_stock > 0:
            return _("In stock (%d available)" % stockrecord.num_in_stock)
        return _("Out of stock")
    
    def dispatch_date(self, stockrecord):
        if stockrecord.num_in_stock:
            # Assume next day for in-stock items
            return datetime.date.today() + datetime.timedelta(days=1)
        # Assume one week for out-of-stock items
        return datetime.date.today() + datetime.timedelta(days=7)
    
    def lead_time(self, stockrecord):
        return 1
    
    def calculate_tax(self, stockrecord):
        return D('0.00')


from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db.models import get_model

from oscar.models.fields import ExtendedURLField

from oscar.core.loading import import_module
catalogue_models = import_module('catalogue.models', ['Product',])

Item = get_model('product', 'Item')

# Linking models


class LinkedPromotion(models.Model):
    
    # We use generic foreign key to link to a promotion model
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    position = models.CharField(_("Position"), max_length=100, help_text="Position on page")
    display_order = models.PositiveIntegerField(default=0)
    clicks = models.PositiveIntegerField(default=0)
    date_created = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True
        ordering = ['-clicks']

    def record_click(self):
        self.clicks += 1
        self.save()


class PagePromotion(LinkedPromotion):
    """
    A promotion embedded on a particular page.
    """
    page_url = ExtendedURLField(max_length=128, db_index=True)

    def __unicode__(self):
        return u"%s on %s" % (self.content_object, self.page_url)
    
    def get_link(self):
        return reverse('promotions:page-click', kwargs={'page_promotion_id': self.id})
        
    
class KeywordPromotion(LinkedPromotion):
    """
    A promotion linked to a specific keyword.

    This can be used on a search results page to show promotions
    linked to a particular keyword.
    """

    keyword = models.CharField(_("Keyword"), max_length=200)
    
    # We allow an additional filter which will let search query matches
    # be restricted to different parts of the site.
    filter = models.CharField(_("Filter"), max_length=200, blank=True, null=True)

    def get_link(self):
        return reverse('promotions:keyword-click', kwargs={'keyword_promotion_id': self.id})


# Different model types for each type of promotion


class AbstractPromotion(models.Model):
    """
    Abstract base promotion that defines the interface
    that subclasses must implement.
    """
    _type = 'Promotion'
    keywords = generic.GenericRelation(KeywordPromotion)
    pages = generic.GenericRelation(PagePromotion)
    
    class Meta:
        abstract = True
    
    @property
    def type(self):
        return _(self._type)
    
    def template_name(self):
        """
        Returns the template to use to render this
        promotion.
        """
        return 'promotions/%s.html' % self.__class__.__name__.lower()
    
    def template_context(self, *args, **kwargs):
        return {}
    
    @property
    def content_type(self):
        return ContentType.objects.get_for_model(self)
    
    @property
    def num_times_used(self):
        ctype = self.content_type
        page_count = PagePromotion.objects.filter(content_type=ctype,
                                                  object_id=self.id).count()
        keyword_count = KeywordPromotion.objects.filter(content_type=ctype,
                                                        object_id=self.id).count()   
        return page_count + keyword_count                                       


class RawHTML(AbstractPromotion):
    """
    Simple promotion - just raw HTML
    """
    _type = 'Raw HTML'
    name = models.CharField(_("Name"), max_length=128)
    body = models.TextField(_("HTML"))
    date_created = models.DateTimeField(auto_now_add=True)

    class Meta:
        verbose_name_plural = 'Raw HTML'
        
    def __unicode__(self):
        return self.name


class Image(AbstractPromotion):
    """
    An image promotion is simply a named image which has an optional 
    link to another part of the site (or another site).
    
    This can be used to model both banners and pods.
    """
    _type = 'Image'
    name = models.CharField(_("Name"), max_length=128)
    link_url = ExtendedURLField(blank=True, null=True, help_text="""This is 
        where this promotion links to""")
    image = models.ImageField(upload_to=settings.OSCAR_PROMOTION_FOLDER)
    date_created = models.DateTimeField(auto_now_add=True)
    
    def __unicode__(self):
        return self.name
    
    
class MultiImage(AbstractPromotion):
    """
    A multi-image promotion is simply a collection of image promotions
    that are rendered in a specific way.  This models things like 
    rotating banners.
    """
    _type = 'Multi-image'
    name = models.CharField(_("Name"), max_length=128)
    images = models.ManyToManyField('promotions.Image', null=True, blank=True)
    date_created = models.DateTimeField(auto_now_add=True)
    
    def __unicode__(self):
        return self.name


class SingleProduct(AbstractPromotion):
    _type = 'Single product'
    name = models.CharField(_("Name"), max_length=128)
    product = models.ForeignKey('catalogue.Product')
    description = models.TextField(_("Description"), null=True, blank=True)
    date_created = models.DateTimeField(auto_now_add=True)
    
    def __unicode__(self):
        return self.name
    

class AbstractProductList(AbstractPromotion):
    """
    Abstract superclass for promotions which are essentially a list
    of products.
    """
    name = models.CharField(_("Title"), max_length=255)
    description = models.TextField(null=True, blank=True)
    link_url = ExtendedURLField(blank=True, null=True)
    date_created = models.DateTimeField(auto_now_add=True)
    
    class Meta:
        abstract = True
        
    def __unicode__(self):
        return self.name
        
 
class HandPickedProductList(AbstractProductList):
    """
    A hand-picked product list is a list of manually selected
    products.
    """
    _type = 'Product list'
    products = models.ManyToManyField('catalogue.Product', through='OrderedProduct', blank=True, null=True)

    def get_products(self):
        return self.products.all().order_by('%s.display_order' % OrderedProduct._meta.db_table)
    

class OrderedProduct(models.Model):
    
    list = models.ForeignKey('promotions.HandPickedProductList')
    product = models.ForeignKey('catalogue.Product')
    display_order = models.PositiveIntegerField(default=0)
    
    class Meta:
        ordering = ('display_order',)
        
        
class AutomaticProductList(AbstractProductList):
    
    _type = 'Auto-product list'
    BESTSELLING, RECENTLY_ADDED = ('Bestselling', 'RecentlyAdded')
    METHOD_CHOICES = (
        (BESTSELLING, _("Bestselling products")),
        (RECENTLY_ADDED, _("Recently added products")),
    )
    method = models.CharField(max_length=128, choices=METHOD_CHOICES)
    num_products = models.PositiveSmallIntegerField(default=4)  
    
    def get_products(self):
        if self.method == self.BESTSELLING:
            return Product.objects.all().order_by('-score')[:self.num_products]
        return Product.objects.all().order_by('-date_created')[:self.num_products]


class OrderedProductList(HandPickedProductList):
    tabbed_block = models.ForeignKey('promotions.TabbedBlock',
                                     related_name='tabs')
    display_order = models.PositiveIntegerField(default=0)

    class Meta:
        ordering = ('display_order',)


class TabbedBlock(AbstractPromotion):

    _type = 'Tabbed block'
    name = models.CharField(_("Title"), max_length=255)
    date_created = models.DateTimeField(auto_now_add=True)




import json

from django.http import HttpResponse, HttpResponseRedirect
from django.views.generic.base import View
from django.conf import settings
from django.template.response import TemplateResponse
from haystack.query import SearchQuerySet
from haystack.views import FacetedSearchView

from oscar.core.loading import import_module
product_models = import_module('catalogue.models', ['Product'])


class SuggestionsView(View):
    u"""
    Auto suggest view

    Returns the suggestions in JSON format (especially suited for consumption by
    jQuery autocomplete)
    """

    suggest_limit = settings.OSCAR_SEARCH_SUGGEST_LIMIT

    def get(self, request):
        context = self.get_context_data()
        return self.render_to_response(context)

    def get_context_data(self):
        '''
        Creates a list of suggestions
        '''
        query_term = self.request.GET['query_term'];
        query_set = SearchQuerySet().filter(text__contains=query_term)[:self.suggest_limit]
        context = []
        for item in query_set:
            context.append({
                'label': item.object.title,
                'url':  item.object.get_absolute_url(),
            })
        return context

    def render_to_response(self, context):
        "Returns a JSON response containing 'context' as payload"
        return self.get_json_response(self.convert_context_to_json(context))

    def get_json_response(self, content, **httpresponse_kwargs):
        "Construct an `HttpResponse` object."
        return HttpResponse(content,
                            content_type='application/json',
                            **httpresponse_kwargs)

    def convert_context_to_json(self, context):
        "Convert the context into a JSON object"
        return json.dumps(context)


class MultiFacetedSearchView(FacetedSearchView):
    u"""
    Search view for multifaceted searches
    """
    template = 'search/results.html'

    def __call__(self, request, *args, **kwargs):
        """
        Generates the actual response to the search.
        
        Relies on internal, overridable methods to construct the response.
        """
        
        # Look for UPC match
        query = request.GET['q'].strip()
        try:
            item = product_models.Product._default_manager.get(upc=query)
            return HttpResponseRedirect(item.get_absolute_url())
        except product_models.Product.DoesNotExist:
            pass
        
        return super(MultiFacetedSearchView, self).__call__(request, *args, **kwargs)

    def __name__(self):
        return "MultiFacetedSearchView"

    def extra_context(self):
        '''
        Adds details about the facets applied
        '''
        extra = super(MultiFacetedSearchView, self).extra_context()

        if hasattr(self.form, 'cleaned_data') and 'selected_facets' in self.form.cleaned_data:
            extra['facets_applied'] = []
            for f in self.form.cleaned_data['selected_facets'].split("|"):
                facet = f.split(":")
                extra['facets_applied'].append({
                    'facet': facet[0][:-6], # removing the _exact suffix that haystack uses for some reason
                    'value' : facet[1].strip('"')
                })
        return extra


from django.contrib import admin

from oscar.core.loading import import_module
import_module('customer.models', ['Email', 'CommunicationEventType'], locals())

admin.site.register(Email)
admin.site.register(CommunicationEventType)





from django.forms import ModelForm
from django.db.models import get_model

UserAddress = get_model('address', 'useraddress')


class UserAddressForm(ModelForm):

    class Meta:
        model = UserAddress
        exclude = ('user', 'num_orders', 'hash', 'search_text')
    


from django.template import Library, Node, Variable, Context
from django.template.loader import select_template
from django.template import RequestContext

     
register = Library()
     
     
class PromotionNode(Node):
    def __init__(self, promotion):
        self.promotion_var = Variable(promotion)
    
    def render(self, context):
        promotion = self.promotion_var.resolve(context)
        template = select_template([promotion.template_name(), 'promotions/default.html'])
        args = {'promotion': promotion}
        args.update(**promotion.template_context(request=context['request']))
        ctx = RequestContext(context['request'], args)
        return template.render(ctx)
 
def get_promotion_html(parser, token):
    _, promotion = token.split_contents()
    return PromotionNode(promotion)

register.tag('render_promotion', get_promotion_html)


from django.contrib.auth.models import User
from django.contrib.auth.backends import ModelBackend


class Emailbackend(ModelBackend):
    def authenticate(self, email=None, password=None, *args, **kwargs):
        if not email:
            email = kwargs.pop('username', None)
        try:
            user = User.objects.get(email=email)
            if user.check_password(password):
                return user
        except User.DoesNotExist:
            return None



from django import template

from oscar.core.loading import import_module
product_models = import_module('catalogue.models', ['Product', 'ProductClass'])
history_helpers = import_module('customer.history_helpers', ['get_recently_viewed_product_ids'])

register = template.Library()

@register.inclusion_tag('customer/history/recently-viewed-products.html', takes_context=True)
def recently_viewed_products(context):
    u"""
    Inclusion tag listing the most recently viewed products
    """
    request = context['request']
    product_ids = history_helpers.get_recently_viewed_product_ids(request)
    product_dict = product_models.Product.browsable.in_bulk(product_ids)
    
    # Reordering as the id order gets messed up in the query
    product_ids.reverse()
    products = [product_dict[id] for id in product_ids if id in product_dict]
    return {'products': products}


from imp import new_module

from django.conf import settings


class AppNotFoundError(Exception):
    pass


def import_module(module_label, classes, namespace=None):
    u"""
    For dynamically importing classes from a module.
    
    Eg. calling import_module('catalogue.models') will search INSTALLED_APPS for
    the relevant product app (default is 'oscar.product') and then import the
    classes from there.  If the class can't be found in the overriding module, 
    then we attempt to import it from within oscar.  
    
    We search the INSTALLED_APPS list to find the appropriate app string and 
    import that.
    
    This is very similar to django.db.models.get_model although that is only 
    for loading models while this method will load any class.
    """
    # Classes must be specified in order for __import__ to work correctly.  It's
    # also a good practice
    if not classes:
        raise ValueError("You must specify the classes to import")
    
    # Arguments will be things like 'catalogue.models' and so we
    # we take the first component to find within the INSTALLED_APPS list.
    app_name = module_label.rsplit(".", 1)[0] 
    for installed_app in settings.INSTALLED_APPS:
        base_package = installed_app.split(".")[0]
        module_name = installed_app.split(".", 2).pop() # strip oscar.apps
        try: 
            # We search the second component of the installed apps
            if app_name == module_name:
                if base_package == 'oscar':
                    # Using core module explicitly
                    return _import_classes_from_module("oscar.apps.%s" % module_label, classes, namespace)
                else:
                    # Using local override - check that requested module exists
                    local_app = "%s.%s" % (base_package, module_label)
                    try:
                        imported_local_mod = __import__(local_app, fromlist=classes)
                    except ImportError, e:
                        # Module doesn't exist, fall back to oscar core.  This can be tricky
                        # as if the overriding module has an import error, it will get picked up
                        # here.
                        if str(e).startswith("No module named"):
                            return _import_classes_from_module("oscar.apps.%s" % module_label, classes, namespace)
                        raise e
                    
                    # Found overriding module, merging custom classes with core
                    module = new_module(local_app)
                    imported_oscar_mod = __import__("oscar.apps.%s" % module_label, fromlist=classes)
                    for classname in classes:
                        if hasattr(imported_local_mod, classname):
                            if namespace:
                                namespace[classname] = getattr(imported_local_mod, classname)
                            else:
                                module.__setattr__(classname, getattr(imported_local_mod, classname))
                        else:
                            if namespace:
                                namespace[classname] = getattr(imported_oscar_mod, classname)
                            else:
                                module.__setattr__(classname, getattr(imported_oscar_mod, classname))
                return module
        except IndexError:
            pass
    raise AppNotFoundError("Unable to find an app matching %s in INSTALLED_APPS" % (app_name,))
    
    
def _import_classes_from_module(module_name, classes, namespace):
    imported_module = __import__(module_name, fromlist=classes)
    if namespace:
        for classname in classes:
            namespace[classname] = getattr(imported_module, classname)
        return 
    
    module = new_module(module_name)   
    for classname in classes:
        setattr(module, classname, getattr(imported_module, classname))
    return module



from django.http import HttpResponseRedirect
from django.views.generic import TemplateView, RedirectView
from django.core.urlresolvers import reverse


class HomeView(TemplateView):
    template_name = 'promotions/home.html'


class RecordClickView(RedirectView):
    
    model=None
    
    def get_redirect_url(self, **kwargs):
        try:
            prom = self.model.objects.get(pk=kwargs['pk'])
        except self.model.DoesNotExist:
            return reverse('promotions:home')  

        if prom.promotion.has_link:
            prom.record_click()
            return prom.link_url
        return reverse('promotions:home')




import zlib

from django.db import models
from django.utils.translation import ugettext_lazy as _


class AbstractAddress(models.Model):
    """
    Superclass address object

    This is subclassed and extended to provide models for 
    user, shipping and billing addresses.

    The only required fields are last_name, line1 and postcode.
    """
    # @todo: Need a way of making these choice lists configurable 
    # per project
    MR, MISS, MRS, MS, DR = ('Mr', 'Miss', 'Mrs', 'Ms', 'Dr')
    TITLE_CHOICES = (
        (MR, _("Mr")),
        (MISS, _("Miss")),
        (MRS, _("Mrs")),
        (MS, _("Ms")),
        (DR, _("Dr")),
    )
    title = models.CharField(_("Title"), max_length=64, choices=TITLE_CHOICES, blank=True, null=True)
    first_name = models.CharField(_("First name"), max_length=255, blank=True, null=True)
    last_name = models.CharField(_("Last name"), max_length=255, blank=True)
    
    # We use quite a few lines of an address as they are often quite long and 
    # it's easier to just hide the unnecessary ones than add extra ones.
    line1 = models.CharField(_("First line of address"), max_length=255)
    line2 = models.CharField(_("Second line of address"), max_length=255, blank=True, null=True)
    line3 = models.CharField(_("Third line of address"), max_length=255, blank=True, null=True)
    line4 = models.CharField(_("City"), max_length=255, blank=True, null=True)
    state = models.CharField(_("State/County"), max_length=255, blank=True, null=True)
    postcode = models.CharField(_("Post/Zip-code"), max_length=64)
    country = models.ForeignKey('address.Country')
    
    # A field only used for searching addresses - this contains all the relevant fields
    search_text = models.CharField(_("Search text"), max_length=1000)
    
    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        self._clean_fields()
        self._update_search_text()
        super(AbstractAddress, self).save(*args, **kwargs)
        
    @property    
    def city(self):
        return self.line4     
        
    def _clean_fields(self):
        """
        Clean up fields
        """
        for field in ['first_name', 'last_name', 'line1', 'line2', 'line3', 'line4', 'postcode']:
            if self.__dict__[field]:
               self.__dict__[field] = self.__dict__[field].strip()
        
        # Ensure postcodes are always uppercase
        if self.postcode:
            self.postcode = self.postcode.upper()
            
    def _update_search_text(self):
        search_fields = filter(lambda x: x, [self.first_name, self.last_name, 
                                             self.line1, self.line2, self.line3, self.line4, self.state,
                                             self.postcode, self.country.name]) 
        self.search_text = ' '.join(search_fields) 
        
    @property    
    def summary(self):
        """
        Returns a single string summary of the address,
        separating fields using commas.
        """
        return u", ".join(self.active_address_fields())    
        
    def populate_alternative_model(self, address_model):
        """
        For populating an address model using the matching fields
        from this one.
        
        This is used to convert a user address to a shipping address
        as part of the checkout process.
        """
        destination_field_names = [field.name for field in address_model._meta.fields]
        for field_name in [field.name for field in self._meta.fields]:
            if field_name in destination_field_names and field_name != 'id':
                setattr(address_model, field_name, getattr(self, field_name))
                
    def active_address_fields(self):
        u"""
        Returns the non-empty components of the address, but merging the
        title, first_name and last_name into a single line.
        """
        self._clean_fields()
        fields = filter(lambda x: x, [self.salutation(), self.line1, self.line2, self.line3,
                                      self.line4, self.state, self.postcode])
        if self.country:
            fields.append(self.country.name)
        return fields
        
    def salutation(self):
        u"""Returns the salutation"""
        return u" ".join([part for part in [self.title, self.first_name, self.last_name] if part])
        
    def name(self):
        """
        Returns the full name
        """
        return u" ".join([part for part in [self.first_name, self.last_name] if part])    
        
    def __unicode__(self):
        return self.summary


class AbstractCountry(models.Model):
    """
    International Organization for Standardization (ISO) 3166-1 Country list.
    """
    iso_3166_1_a2 = models.CharField(_('ISO 3166-1 alpha-2'), max_length=2, primary_key=True)
    iso_3166_1_a3 = models.CharField(_('ISO 3166-1 alpha-3'), max_length=3, null=True, db_index=True)
    iso_3166_1_numeric = models.PositiveSmallIntegerField(_('ISO 3166-1 numeric'), null=True, db_index=True)
    name = models.CharField(_('Official name (CAPS)'), max_length=128)
    printable_name = models.CharField(_('Country name'), max_length=128)
    
    is_highlighted = models.BooleanField(default=False, db_index=True)
    is_shipping_country = models.BooleanField(default=False, db_index=True)
    
    class Meta:
        abstract = True
        verbose_name = _('Country')
        verbose_name_plural = _('Countries')
        ordering = ('-is_highlighted', 'name',)
            
    def __unicode__(self):
        return self.printable_name
        

class AbstractShippingAddress(AbstractAddress):
    u"""
    Shipping address.
    
    A shipping address should not be edited once the order has been placed - 
    it should be read-only after that. 
    """
    phone_number = models.CharField(max_length=32, blank=True, null=True)
    notes = models.TextField(blank=True, null=True, help_text="""Shipping notes""")
    
    class Meta:
        abstract = True
        verbose_name_plural = "shipping addresses"
        
    @property    
    def order(self):
        """
        Return the order linked to this shipping address
        """
        orders = self.order_set.all()
        if not orders:
            return None
        return orders[0]
        
        
class AbstractUserAddress(AbstractShippingAddress):
    """
    A user address which forms an "AddressBook" for a user.
    
    We use a separate model to shipping and billing (even though there will be
    some data duplication) because we don't want shipping/billing addresses changed
    or deleted once an order has been placed.  By having a separate model, we allow
    users the ability to add/edit/delete from their address book without affecting
    orders already placed. 
    """
    user = models.ForeignKey('auth.User', related_name='addresses')
    
    # Customers can set defaults
    is_default_for_shipping = models.BooleanField(default=False)
    is_default_for_billing = models.BooleanField(default=False)
    
    # We keep track of the number of times an address has been used
    # as a shipping address so we can show the most popular ones 
    # first at the checkout.
    num_orders = models.PositiveIntegerField(default=0)
    
    # A hash is kept to try and avoid duplicate addresses being added
    # to the address book.
    hash = models.CharField(max_length=255, db_index=True)
    date_created = models.DateTimeField(auto_now_add=True)
    
    def generate_hash(self):
        u"""Returns a hash of the address summary."""
        # We use an upper-case version of the summary
        return zlib.crc32(self.summary.strip().upper().encode('UTF8'))

    def save(self, *args, **kwargs):
        u"""Save a hash of the address fields"""
        # Save a hash of the address fields so we can check whether two 
        # addresses are the same to avoid saving duplicates
        self.hash = self.generate_hash()
        super(AbstractUserAddress, self).save(*args, **kwargs)
    
    class Meta:
        abstract = True
        verbose_name_plural = "User addresses"
        ordering = ['-num_orders']


class AbstractBillingAddress(AbstractAddress):
    
    class Meta:
        abstract = True
        verbose_name_plural = "Billing addresses"   
        
    @property    
    def order(self):
        """
        Return the order linked to this shipping address
        """
        orders = self.order_set.all()
        if not orders:
            return None
        return orders[0] 



import datetime

from django.test import TestCase
from django.contrib.auth.models import User

from oscar.apps.payment.models import Bankcard
from oscar.apps.payment.forms import (bankcard_type, VISA, MASTERCARD, AMEX, DISCOVER, 
                                      BankcardNumberField, BankcardExpiryMonthField, luhn)


class BankcardTest(TestCase):
    
    def test_get_obfuscated_number(self):
        bankcard = Bankcard(name="David Winterbottom", number="1000011100000004")
        self.assertEquals("XXXX-XXXX-XXXX-0004", bankcard._get_obfuscated_number())
    
    def test_number_is_anonymised_when_saving(self):
        user = User.objects.create(username='Dummy user')
        expiry_date = datetime.date(year=2012, month=02, day=12)
        bankcard = Bankcard.objects.create(name="David Winterbottom", number="1000011100000004", user=user, expiry_date=expiry_date)
        self.assertEquals("XXXX-XXXX-XXXX-0004", bankcard.number)


class BankcardTypeTest(TestCase):
    
    fixture_data = {
        VISA: ('4111111111111111',),
        MASTERCARD: ('5500000000000004',),
        DISCOVER: ('6011000000000004',),
        AMEX: ('340000000000009',),
    }
    
    def test_bankcard_type_sniffer(self):
        for type, numbers in self.fixture_data.items():
            for n in numbers:
                self.assertEquals(type, bankcard_type(n), "%s is a %s" % (n, type))
        

class BankcardNumberFieldTest(TestCase):
    
    def setUp(self):
        self.f = BankcardNumberField()
        
    def test_spaces_are_stipped(self):
        self.assertEquals('4111111111111111', self.f.clean('  4111 1111 1111 1111'))
        

class LuhnTest(TestCase):
    
    valid_numbers = ['4111111111111111',
                     '5500000000000004',
                     '6011000000000004',
                     '340000000000009']
    
    # Tweaked versions of above valid numbers
    invalid_numbers = ['4111111111111110',
                     '5500000000000009',
                     '6011000000000000',
                     '340000000000005']
    
    def test_valid_numbers_pass(self):
        for number in self.valid_numbers:
            self.assertTrue(luhn(number))
            
    def test_invalid_numbers_fail(self):
        for number in self.invalid_numbers:
            self.assertFalse(luhn(number))
    
    


from django.conf.urls.defaults import patterns, url

from oscar.apps.basket.views import BasketView, SavedView, VoucherView, VoucherAddView, BasketAddView, VoucherRemoveView
from oscar.core.application import Application


class BasketApplication(Application):
    name = 'basket'
    summary_view = BasketView
    saved_view = SavedView
    voucher_view = VoucherView
    add_view = BasketAddView
    add_voucher_view = VoucherAddView
    remove_voucher_view = VoucherRemoveView

    def get_urls(self):
        urlpatterns = patterns('',                   
            url(r'^$', self.summary_view.as_view(), name='summary'),
            url(r'^add/$', self.add_view.as_view(), name='add'),    
            url(r'^vouchers/$', self.voucher_view.as_view(), name='vouchers'),
            url(r'^vouchers/add/$', self.add_voucher_view.as_view(), name='vouchers-add'),
            url(r'^vouchers/(?P<pk>\d+)/remove/$', self.remove_voucher_view.as_view(), name='vouchers-remove'),     
            url(r'^saved/$', self.saved_view.as_view(), name='saved'),
        )
        return urlpatterns

application = BasketApplication()


from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.generic import ListView, DetailView, CreateView
from django.db.models import Avg
from django.contrib import messages
from oscar.apps.catalogue.reviews.forms import SignedInUserProductReviewForm, AnonymousUserProductReviewForm, VoteForm
from django.db.models import get_model

vote_model = get_model('reviews', 'vote')


class CreateProductReview(CreateView):
    template_name = "reviews/add_review.html"
    model = get_model('reviews', 'productreview')
    product_model = get_model('catalogue', 'product')
    review_form = SignedInUserProductReviewForm
    anonymous_review_form = AnonymousUserProductReviewForm
    
    def get(self, request, *args, **kwargs):
        if request.user.is_authenticated():
            product = self.get_product()
            try:
                self.model.objects.get(user=request.user, product=product)
                messages.info(self.request, "You have already reviewed this product!")
                return HttpResponseRedirect(product.get_absolute_url()) 
            except self.model.DoesNotExist:
                pass
        return super(CreateProductReview, self).get(request, *args, **kwargs)        
    
    def get_context_data(self, **kwargs):
        context = super(CreateProductReview, self).get_context_data(**kwargs)
        context['product'] = self.get_product()
        return context
    
    def get_product(self):
        return get_object_or_404(self.product_model, pk=self.kwargs['product_pk'])
    
    def get_form_class(self):
        if not self.request.user.is_authenticated():
            return self.anonymous_review_form
        return self.review_form
    
    def get_form_kwargs(self):
        kwargs = super(CreateProductReview, self).get_form_kwargs()
        review = self.model(product=self.get_product())
        if self.request.user.is_authenticated():
            review.user = self.request.user
        kwargs['instance'] = review
        return kwargs
    
    def get_success_url(self):
        return self.object.product.get_absolute_url()


class CreateProductReviewComplete(DetailView):
    template_name = "reviews/add_review_complete.html"
    context_object_name = 'review'
    model = get_model('reviews', 'productreview')
    product_model = get_model('catalogue', 'product')
    
    def get_context_data(self, **kwargs):
        context = super(CreateProductReviewComplete, self).get_context_data(**kwargs)
        context['product'] = get_object_or_404(self.product_model, pk=self.kwargs['product_pk'])
        return context    


class ProductReviewDetail(DetailView):
    """
    Places each review on its own page
    """
    
    template_name = "reviews/review.html"
    context_object_name = 'review'
    model = get_model('reviews', 'productreview')
    product_model = get_model('catalogue', 'product')
    vote_model = vote_model
    
    def get_context_data(self, **kwargs):
        context = super(ProductReviewDetail, self).get_context_data(**kwargs)
        context['product'] = get_object_or_404(self.product_model, pk=self.kwargs['product_pk'])
        return context
    
    def post(self, request, *args, **kwargs ):
        review = self.get_object()
        try:
            vote = self.vote_model.objects.get(user=request.user, review=review)
        except self.vote_model.DoesNotExist:
            vote = self.vote_model(user=request.user, review=review)
        form = VoteForm(request.POST,instance=vote)

        if form.is_valid():
            form.save()
            messages.info(request, "Thanks for voting!")
        else:
            messages.info(request, "We couldn't process your vote")
        return HttpResponseRedirect(request.META.get('HTTP_REFERER',review.get_absolute_url()))

    
class ProductReviewList(ListView):
    u"""A list of reviews for a particular product
    * The review browsing page allows reviews to be sorted by score, or recency.
    """    
    template_name = 'reviews/reviews.html'
    context_object_name = "reviews"
    model = get_model('reviews', 'productreview')
    product_model = get_model('catalogue', 'product')    
    paginate_by = 20
     
    def get_queryset(self):
        qs = self.model.approved.filter(product=self.kwargs['product_pk'])
        if 'sort_by' in self.request.GET and self.request.GET['sort_by'] == 'score':
            return qs.order_by('-score')
        return qs.order_by('-date_created')
     
    def get_context_data(self, **kwargs):
        context = super(ProductReviewList, self).get_context_data(**kwargs)
        context['product'] = get_object_or_404(self.product_model, pk=self.kwargs['product_pk'])  
        context['avg_score'] = self.object_list.aggregate(Avg('score'))           
        return context



u"""Vanilla implementation of order models"""
from django.db import models

from oscar.apps.order.abstract_models import *
from oscar.apps.address.abstract_models import AbstractShippingAddress, AbstractBillingAddress

class Order(AbstractOrder):
    pass

class OrderNote(AbstractOrderNote):
    pass

class CommunicationEvent(AbstractCommunicationEvent):
    pass

class ShippingAddress(AbstractShippingAddress):
    pass

class BillingAddress(AbstractBillingAddress):
    pass
    
class Line(AbstractLine):
    pass

class LinePrice(AbstractLinePrice):
    pass

class LineAttribute(AbstractLineAttribute):
    pass

class ShippingEvent(AbstractShippingEvent):
    pass

class ShippingEventType(AbstractShippingEventType):
    pass

class PaymentEvent(AbstractPaymentEvent):
    pass

class PaymentEventType(AbstractPaymentEventType):
    pass

class OrderDiscount(AbstractOrderDiscount):
    pass


    


from django import template
from django.conf import settings
from django.db.models.fields.files import ImageFieldFile

register = template.Library()


def do_dynamic_image_url(parser, token):
    tokens = token.split_contents()

    if len(tokens) < 2:
        raise template.TemplateSyntaxError("%r tag requires at least an image URL or field" % tokens[0])

    image = tokens[1]

    if len(tokens) > 2:
        params = tokens[2:]
    else:
        params = []
    return DynamicImageNode(image, params)


class DynamicImageNode(template.Node):
    def __init__(self, image, params):
        self.image = image
        self.params = {}

        for p in params:
            try:
                bits = p.split('=')
                self.params[bits[0]] = template.Variable(bits[1])
            except IndexError:
                raise template.TemplateSyntaxError("image tag parameters must be of form key=value, you used '%s'" % p)

    def render(self, context):
        if isinstance(self.image, ImageFieldFile):
            path = self.image.name
        else:
            path = self.image

        host = getattr(settings,'DYNAMIC_MEDIA_URL', None)

        if host:
            params = []

            ext = path[path.rfind('.') + 1:]
            ext_changed = False
    
            for key, v in self.params.iteritems():
                value = v.resolve(context)
    
                if key == u'format':
                    ext = value
                    ext_changed = True
                else:
                    params.append('%s-%s' % (key, value))
    
            if len(params) > 0:
                suffix = '_'.join(params)
                path = '.'.join((path, suffix, ext))
            else:
                if ext_changed:
                    if params:
                        path = '.'.join((path, ext))
                    else:
                        path = '.'.join((path, 'to', ext))

    
            return host + path

register.tag('image', do_dynamic_image_url)



import logging

from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.conf import settings

from oscar.core.loading import import_module
import_module('order.models', ['CommunicationEvent',], locals())
import_module('customer.models', ['Email'], locals())


class Dispatcher(object):

    def __init__(self, logger=None):
        if not logger:
            logger = logging.getLogger(__name__)
        self.logger = logger

    # Public API methods

    def dispatch_direct_messages(self, recipient, messages):
        """
        Dispatch one-off messages to explicitly specified recipient(s).
        """
        if messages['subject'] and messages['body']:
            self.send_email_messages(recipient, messages)
    
    def dispatch_order_messages(self, order, messages, event_type):
        """
        Dispatch order-related messages to the customer
        """
        self.dispatch_user_messages(order.user, messages)
            
        # Create order comms event for audit
        if event_type:
            CommunicationEvent._default_manager.create(order=order, type=event_type)
    
    def dispatch_user_messages(self, user, messages):
        """
        Send messages to a site user
        """
        if messages['subject'] and messages['body']:
            self.send_user_email_messages(user, messages)
        if messages['sms']:
            self.send_text_message(user, messages['sms'])
    
    # Internal
    
    def send_user_email_messages(self, user, messages):
        """
        Sends message to the registered user / customer and collects data in database
        """
        if not user.email:
            self.logger.warning("Unable to send email messages as user #%d has no email address", user.id)
            return
        
        email = self.send_email_messages(user.email, messages)
        
        # Is user is signed in, record the event for audit
        if email and user.is_authenticated():
            Email._default_manager.create(user=user, 
                                          subject=email.subject,
                                          body_text=email.body,
                                          body_html=messages['html'])
    
    def send_email_messages(self, recipient, messages):
        """
        Plain email sending to the specified recipient
        """
        if hasattr(settings, 'OSCAR_FROM_EMAIL'):
            from_email = settings.OSCAR_FROM_EMAIL
        else:
            from_email = None
            
        # Determine whether we are sending a HTML version too
        if messages['html']:
            email = EmailMultiAlternatives(messages['subject'],
                                           messages['body'],
                                           from_email=from_email,
                                           to=[recipient])
            email.attach_alternative(messages['html'], "text/html")
        else:
            email = EmailMessage(messages['subject'],
                                 messages['body'],
                                 from_email=from_email,
                                 to=[recipient])
        self.logger.info("Sending email to %s" % recipient)
        email.send()
        
        return email
        
    def send_text_message(self, user, event_type):
        raise NotImplementedError




from django.dispatch import receiver
from django.db import IntegrityError
import logging

from oscar.core.loading import import_module
import_module('analytics.models', ['UserRecord', 'ProductRecord', 'UserProductView'], locals())
import_module('catalogue.signals', ['product_viewed', 'product_search'], locals())
import_module('basket.signals', ['basket_addition'], locals())
import_module('order.signals', ['order_placed'], locals())

# Helpers

logger = logging.getLogger('oscar.analytics')

def _record_product_view(product):
    try:
        record,_ = ProductRecord.objects.get_or_create(product=product)
        record.num_views += 1
        record.save()
    except IntegrityError:
        # get_or_create sometimes fails due to MySQL's weird transactions, fail silently
        logger.error("IntegrityError on ProductRecord.objects.get_or_create(product=product)")
    
def _record_user_product_view(user, product):
    if user.is_authenticated():
        # Update user record
        try:
            record,_ = UserRecord.objects.get_or_create(user=user)
            record.num_product_views += 1
            record.save()
        except IntegrityError:
            logger.error("IntegrityError on UserRecord.objects.get_or_create(user=user)")
        
        # Add user product view record
        UserProductView.objects.create(product=product, user=user)

def _record_basket_addition(product):
    try:
        record,_ = ProductRecord.objects.get_or_create(product=product)
        record.num_basket_additions += 1
        record.save()
    except IntegrityError:
        logger.error("IntegrityError on ProductRecord.objects.get_or_create(product=product)")
    
def _record_user_basket_addition(user, product):
    if user.is_authenticated():
        try:
            record,_ = UserRecord.objects.get_or_create(user=user)
            record.num_basket_additions += 1
            record.save()
        except IntegrityError:
            logger.error("IntegrityError on UserRecord.objects.get_or_create(user=user)")

def _record_products_in_order(order):
    for line in order.lines.all():
        try:
            record,_ = ProductRecord.objects.get_or_create(product=line.product)
            record.num_purchases += line.quantity
            record.save()
        except IntegrityError:
            logger.error("IntegrityError on ProductRecord.objects.get_or_create(product=product)")

def _record_user_order(user, order):
    if user.is_authenticated():
        try:
            record,_ = UserRecord.objects.get_or_create(user=user)
            record.num_orders += 1
            record.num_order_lines += order.num_lines
            record.num_order_items += order.num_items
            record.total_spent += order.total_incl_tax
            record.date_last_order = order.date_placed
            record.save()
        except IntegrityError:
            logger.error("IntegrityError on UserRecord.objects.get_or_create(user=user)")

def _record_user_product_search(user, query):
    if user.is_authenticated():
        UserSearch._default_manager.create(user=user, query=query)

# Receivers

@receiver(product_viewed)
def receive_product_view(sender, product, user, **kwargs):
    _record_product_view(product)
    _record_user_product_view(user, product)
    
@receiver(product_search)
def receive_product_search(sender, query, user, **kwargs):
    _record_user_product_search(user, query)
    
@receiver(basket_addition)
def receive_basket_addition(sender, product, user, **kwargs):
    _record_basket_addition(product)
    _record_user_basket_addition(user, product)
    
@receiver(order_placed)
def receive_order_placed(sender, order, user, **kwargs):
    _record_products_in_order(order)
    _record_user_order(user, order)



import os
import zlib
import tarfile
import zipfile
import tempfile
import shutil
from PIL import Image as PImage

from django.core.files import File
from django.core.exceptions import FieldError
from django.db.models import get_model

from oscar.core.loading import import_module
import_module('catalogue.exceptions', ['ImageImportError', 'IdenticalImageError', 'InvalidImageArchive'], locals())

Category = get_model('catalogue', 'category')
Product = get_model('catalogue', 'product')
ProductImage = get_model('catalogue', 'productimage')


def create_categories(bits):
    if len(bits) == 1:
        # Get or create root node
        try:
            root = Category.objects.get(depth=1, name=bits[0])
        except Category.DoesNotExist:
            root = Category.add_root(name=bits[0])
        return [root]
    else:
        parents = create_categories(bits[:-1])
        try:
            child = parents[-1].get_children().get(name=bits[-1])
        except Category.DoesNotExist:
            child = parents[-1].add_child(name=bits[-1])
        parents.append(child)
        return parents

def breadcrumbs_to_category(breadcrumbs, separator='>'):
    bits = [x.strip() for x in breadcrumbs.split(separator)]
    categories = create_categories(bits)
    return categories[-1]


class Importer(object):
    
    allowed_extensions = ['.jpeg','.jpg','.gif','.png']
    
    def __init__(self, logger, field):
        self.logger = logger
        self._field = field
        
    def handle(self, dirname):
        stats = {
            'num_processed': 0,
            'num_skipped': 0,
            'num_invalid': 0         
        }
        image_dir, filenames = self._get_image_files(dirname)
        if image_dir:
            for filename in filenames:
                try:
                    lookup_value = self._get_lookup_value_from_filename(filename)
                    self._process_image(image_dir, filename, lookup_value)
                    stats['num_processed'] += 1
                except Product.MultipleObjectsReturned:
                    self.logger.warning("Multiple products matching %s='%s', skipping" % (self._field, lookup_value))
                    stats['num_skipped'] += 1
                except Product.DoesNotExist:
                    self.logger.warning("No item matching %s='%s'" % (self._field, lookup_value))
                    stats['num_skipped'] += 1
                except IdenticalImageError:
                    self.logger.warning(" - Identical image already exists for %s='%s', skipping" % (self._field, lookup_value))
                    stats['num_skipped'] += 1
                except IOError, e:
                    raise ImageImportError('%s is not a valid image (%s)' % (filename, e))    
                    stats['num_invalid'] += 1
                except FieldError, e:
                    raise ImageImportError(e)
                    self._process_image(image_dir, filename)
            if image_dir != dirname:
                shutil.rmtree(image_dir)
        else:
            raise InvalidImageArchive('%s is not a valid image archive' % dirname)
        self.logger.info("Finished image import: %(num_processed)d imported, %(num_skipped)d skipped" % stats)
        
    def _get_image_files(self, dirname):
        filenames = []
        image_dir = self._extract_images(dirname)
        if image_dir:
            for filename in os.listdir(image_dir):
                ext = os.path.splitext(filename)[1]
                if os.path.isfile(os.path.join(image_dir, filename)) and ext in self.allowed_extensions:
                    filenames.append(filename)
        return image_dir, filenames

    def _extract_images(self, dirname):
        '''
        Returns path to directory containing images in dirname if successful.
        Returns empty string if dirname does not exist, or could not be opened.
        Assumes that if dirname is a directory, then it contains images.
        If dirname is an archive (tar/zip file) then the path returned is to a 
        temporary directory that should be deleted when no longer required.
        '''
        if os.path.isdir(dirname):
            return dirname        

        ext = os.path.splitext(dirname)[1]
        if ext in ['.gz', '.tar']:
            image_dir = tempfile.mkdtemp()
            try:
                tar_file = tarfile.open(dirname)
                tar_file.extractall(image_dir)
                tar_file.close()
                return image_dir
            except (tarfile.TarError, zlib.error):
                return ""
        elif ext == '.zip':
            image_dir = tempfile.mkdtemp()
            try:
                zip_file = zipfile.ZipFile(dirname)
                zip_file.extractall(image_dir) 
                zip_file.close()               
                return image_dir
            except (zlib.error, zipfile.BadZipfile, zipfile.LargeZipFile):
                return ""
        # unknown archive - perhaps this should be treated differently
        return ""
                
    def _process_image(self, dirname, filename, lookup_value):
        file_path = os.path.join(dirname, filename)
        trial_image = PImage.open(file_path)
        trial_image.verify()
        
        kwargs = {self._field: lookup_value}
        item = Product._default_manager.get(**kwargs)
        
        new_data = open(file_path).read()
        next_index = 0
        for existing in item.images.all():
            next_index = existing.display_order + 1
            try:
                if new_data == existing.original.read():
                    raise IdenticalImageError()
            except IOError:
                # File probably doesn't exist
                existing.delete()
            
        new_file = File(open(file_path))
        im = ProductImage(product=item, display_order=next_index)
        im.original.save(filename, new_file, save=False)
        im.save()
        self.logger.info(' - Image added to "%s"' % item)
        
    def _fetch_item(self, filename):
        kwargs = {self._field: self._get_lookup_value_from_filename(filename)}
        return Product._default_manager.get(**kwargs)
    
    def _get_lookup_value_from_filename(self, filename):
        return os.path.splitext(filename)[0]
        



import mock
from decimal import Decimal as D
from xml.dom.minidom import parseString

from django.test import TestCase

from oscar.apps.payment.datacash.utils import Gateway


class AuthResponseHandlingTests(TestCase):
    
    success_response_xml = """<?xml version="1.0" encoding="UTF-8" ?>
<Response>
    <CardTxn>
        <authcode>060642</authcode>
        <card_scheme>Switch</card_scheme>
        <country>United Kingdom</country>
        <issuer>HSBC</issuer>
    </CardTxn>
    <datacash_reference>3000000088888888</datacash_reference>
    <merchantreference>1000001</merchantreference>
    <mode>LIVE</mode>
    <reason>ACCEPTED</reason>
    <status>1</status>
    <time>1071567305</time>
</Response>"""
    
    def setUp(self):
        self.gateway = Gateway(client="DUMMY", password="123456", host="dummyhost.com",)
        self.gateway.do_request = mock.Mock()
    
    def test_success_auth_response(self):
        self.gateway.do_request.return_value = self.success_response_xml
        response = self.gateway.auth(card_number='1000010000000007', 
                                     expiry_date='01/12',
                                     merchant_reference='1000001',
                                     currency='GBP',
                                     amount=D('12.99'))
        self.assertEquals(1, response['status'])
        self.assertEquals('3000000088888888', response['datacash_reference'])
        self.assertEquals('1000001', response['merchant_reference'])
        self.assertEquals('060642', response['auth_code'])
        self.assertEquals('ACCEPTED', response['reason'])
        
    def test_success_pre_response(self):
        self.gateway.do_request.return_value = self.success_response_xml
        response = self.gateway.pre(card_number='1000010000000007', 
                                     expiry_date='01/12',
                                     merchant_reference='1000001',
                                     currency='GBP',
                                     amount=D('12.99'))
        self.assertEquals(1, response['status'])
        self.assertEquals('3000000088888888', response['datacash_reference'])
        self.assertEquals('1000001', response['merchant_reference'])
        self.assertEquals('060642', response['auth_code'])
        self.assertEquals('ACCEPTED', response['reason'])
        
    def test_declined_auth_response(self):
        response_xml = """<?xml version="1.0" encoding="UTF-8" ?>
<Response>
    <CardTxn>
        <authcode>DECLINED</authcode>
        <card_scheme>Mastercard</card_scheme>
        <country>United Kingdom</country>
    </CardTxn>
    <datacash_reference>4400200045583767</datacash_reference>
    <merchantreference>AA004630</merchantreference>
    <mode>TEST</mode>
    <reason>DECLINED</reason>
    <status>7</status>
    <time>1169223906</time>
</Response>"""
        self.gateway.do_request.return_value = response_xml
        response = self.gateway.auth(card_number='1000010000000007', 
                                     expiry_date='01/12',
                                     merchant_reference='1000001',
                                     currency='GBP',
                                     amount=D('12.99'))
        self.assertEquals(7, response['status'])
        self.assertEquals('4400200045583767', response['datacash_reference'])
        self.assertEquals('AA004630', response['merchant_reference'])
        self.assertEquals('DECLINED', response['auth_code'])
        self.assertEquals('DECLINED', response['reason'])

    def test_successful_cancel_response(self):
        response_xml = """<?xml version="1.0" encoding="UTF-8" ?>
<Response>
    <datacash_reference>4900200000000001</datacash_reference>
    <merchantreference>4900200000000001</merchantreference>
    <mode>TEST</mode>
    <reason>CANCELLED OK</reason>
    <status>1</status>
    <time>1151567456</time>
</Response>"""
        self.gateway.do_request.return_value = response_xml
        response = self.gateway.cancel(txn_reference='4900200000000001')
        self.assertEquals(1, response['status'])
        self.assertEquals('4900200000000001', response['datacash_reference'])
        self.assertEquals('4900200000000001', response['merchant_reference'])
        self.assertEquals('CANCELLED OK', response['reason'])
        
    def test_successful_fulfil_response(self):
        response_xml = """<?xml version="1.0" encoding="UTF-8" ?>
<Response>
    <datacash_reference>3900200000000001</datacash_reference>
    <merchantreference>3900200000000001</merchantreference>
    <mode>LIVE</mode>
    <reason>FULFILLED OK</reason>
    <status>1</status>
    <time>1071567356</time>
</Response>
"""
        self.gateway.do_request.return_value = response_xml
        response = self.gateway.fulfil(txn_reference='3900200000000001', 
                                     merchant_reference='1000001',
                                     currency='GBP',
                                     amount=D('12.99'),
                                     auth_code='asdf')
        self.assertEquals(1, response['status'])
        self.assertEquals('3900200000000001', response['datacash_reference'])
        self.assertEquals('3900200000000001', response['merchant_reference'])
        self.assertEquals('FULFILLED OK', response['reason'])



        
    
 


import locale
from decimal import Decimal

from django import template
from django.conf import settings

register = template.Library()

@register.filter(name='currency')
def currency(value):
    u"""Return value converted to a locale currency"""
    try:
        locale.setlocale(locale.LC_ALL, settings.LOCALE)
    except AttributeError:
        locale.setlocale(locale.LC_ALL, '')
        
    # We allow the currency symbol to be overridden    
    symbol = getattr(settings, 'CURRENCY_SYMBOL', None)
    try:
        if symbol:
            return "%s%s" % (symbol, locale.format("%.2f", value, grouping=True))
        else:
            return locale.currency(value, symbol=True, grouping=True)
    except TypeError:
        return '' 
        



from django.contrib import admin
from django.db.models import get_model

class UserAddressAdmin(admin.ModelAdmin):
    exclude = ('search_text', 'hash')
    readonly_fields = ('num_orders',)


admin.site.register(get_model('address', 'useraddress'), UserAddressAdmin)
admin.site.register(get_model('address', 'country'))



# Basket settings
OSCAR_BASKET_COOKIE_LIFETIME = 7*24*60*60
OSCAR_BASKET_COOKIE_OPEN = 'oscar_open_basket'
OSCAR_BASKET_COOKIE_SAVED = 'oscar_saved_basket'

# Currency
OSCAR_DEFAULT_CURRENCY = 'GBP'

# Max number of products to keep on the user's history
OSCAR_RECENTLY_VIEWED_PRODUCTS = 4

# Image paths
OSCAR_IMAGE_FOLDER = 'images/products/%Y/%m/'
OSCAR_PROMOTION_FOLDER = 'images/promotions/'

# Search settings
OSCAR_SEARCH_SUGGEST_LIMIT = 10

# Checkout
OSCAR_ALLOW_ANON_CHECKOUT = False

# Partners
OSCAR_PARTNER_WRAPPERS = {}

# Promotions
COUNTDOWN, LIST, SINGLE_PRODUCT, TABBED_BLOCK = ('Countdown', 'List', 'SingleProduct', 'TabbedBlock')
OSCAR_PROMOTION_MERCHANDISING_BLOCK_TYPES = (
    (COUNTDOWN, "Vertical list"),
    (LIST, "Horizontal list"),
    (TABBED_BLOCK, "Tabbed block"),
    (SINGLE_PRODUCT, "Single product"),
)

# Reviews
OSCAR_ALLOW_ANON_REVIEWS = True
OSCAR_MODERATE_REVIEWS = False

# Registration
OSCAR_SEND_REGISTRATION_EMAIL = True
OSCAR_FROM_EMAIL = 'oscar@example.com'



"""
Installation script:

To release a new version to PyPi:
- Ensure the version is correctly set in oscar.__init__.py
- Run: python setup.py sdist upload
"""

from setuptools import setup

from oscar import get_version

setup(name='django-oscar',
      version=get_version().replace(' ', '-'),
      url='https://github.com/tangentlabs/django-oscar',
      author="Tangent Labs",
      author_email="david.winterbottom@tangentlabs.co.uk",
      description="A domain-driven e-commerce framework for Django 1.3+",
      long_description=open('README.rst').read(),
      license='BSD',
      platforms=['linux'],
      packages=['oscar',
                'oscar.apps',
                'oscar.apps.address',
                'oscar.apps.analytics',
                'oscar.apps.analytics.management',
                'oscar.apps.analytics.management.commands',
                'oscar.apps.basket',
                'oscar.apps.basket.templatetags',
                'oscar.apps.catalogue',
                'oscar.apps.catalogue.management',
                'oscar.apps.catalogue.management.commands',
                'oscar.apps.catalogue.reviews',
                'oscar.apps.checkout',
                'oscar.apps.customer',
                'oscar.apps.customer.templatetags',
                'oscar.apps.discount',
                'oscar.apps.dynamic_images',
                'oscar.apps.dynamic_images.templatetags',
                'oscar.apps.offer',
                'oscar.apps.order',
                'oscar.apps.order_management',
                'oscar.apps.partner',
                'oscar.apps.partner.management',
                'oscar.apps.partner.management.commands',
                'oscar.apps.partner.tests',
                'oscar.apps.payment',
                'oscar.apps.payment.datacash',
                'oscar.apps.payment.tests',
                'oscar.apps.promotions',
                'oscar.apps.promotions.templatetags',
                'oscar.apps.reports',
                'oscar.apps.search',
                'oscar.apps.shipping',
                'oscar.apps.voucher',
                'oscar.core',
                'oscar.core.logging',
                'oscar.forms',
                'oscar.templatetags',
                'oscar.test',
                'oscar.views',
                'oscar.models',],
      package_data={'oscar': ['templates/basket/*.html',
                              'templates/catalogue/*.html',
                              'templates/checkout/*.html',
                              'templates/customer/*.html',
                              'templates/customer/history/*.html',
                              'templates/order_management/*.html',
                              'templates/promotions/*.html',
                              'templates/reports/*.html',
                              'templates/reviews/*.html',
                              'templates/search/*.html',
                              'templates/search/indexes/*/*.txt',],
                    'oscar.apps.address': ['fixtures/*'],
                    'oscar.apps.catalogue': ['fixtures/*'],
                    'oscar.apps.catalogue.reviews': ['fixtures/*'],
                    'oscar.apps.dynamic_images': ['test_fixtures/*'],
                    'oscar.apps.offer': ['fixtures/*'],
                    'oscar.apps.order': ['fixtures/*'],
                    'oscar.apps.partner': ['fixtures/*'],
                    'oscar.apps.search': ['fixtures/*'],
                    'oscar.apps.shipping': ['fixtures/*'],},
      install_requires=[
          'django-extra-views==0.1.0',
          'django-haystack>=1.2.0',
          'django-treebeard>=1.6.1',
          'sorl-thumbnail>=11.05.1',
          ],
      dependency_links = [
          'http://github.com/AndrewIngram/django-extra-views/tarball/master#egg=django-extra-views-0.1.0',
      ],
      # See http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=['Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: Unix',
                   'Programming Language :: Python']
      )
